# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import re  # noqa: F401
import io
import warnings

from pydantic.v1 import validate_arguments, ValidationError
from typing_extensions import Annotated
from typing import overload, Optional, Union, Awaitable

from pydantic.v1 import Field, StrictStr

from typing import Optional

from linebot.v3.moduleattach.models.attach_module_response import AttachModuleResponse

from linebot.v3.moduleattach.async_api_client import AsyncApiClient
from linebot.v3.moduleattach.api_response import ApiResponse
from linebot.v3.moduleattach.exceptions import (  # noqa: F401
    ApiTypeError,
    ApiValueError
)


class AsyncLineModuleAttach(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = AsyncApiClient.get_default()
        self.api_client = api_client
        self.line_base_path = "https://manager.line.biz"


    @overload
    async def attach_module(self, grant_type : Annotated[StrictStr, Field(..., description="authorization_code")], code : Annotated[StrictStr, Field(..., description="Authorization code received from the LINE Platform.")], redirect_uri : Annotated[StrictStr, Field(..., description="Specify the redirect_uri specified in the URL for authentication and authorization.")], code_verifier : Annotated[Optional[StrictStr], Field(description="Specify when using PKCE (Proof Key for Code Exchange) defined in the OAuth 2.0 extension specification as a countermeasure against authorization code interception attacks.")] = None, client_id : Annotated[Optional[StrictStr], Field(description="Instead of using Authorization header, you can use this parameter to specify the channel ID of the module channel. You can find the channel ID of the module channel in the LINE Developers Console. ")] = None, client_secret : Annotated[Optional[StrictStr], Field(description="Instead of using Authorization header, you can use this parameter to specify the channel secret of the module channel. You can find the channel secret of the module channel in the LINE Developers Console. ")] = None, region : Annotated[Optional[StrictStr], Field(description="If you specified a value for region in the URL for authentication and authorization, specify the same value. ")] = None, basic_search_id : Annotated[Optional[StrictStr], Field(description="If you specified a value for basic_search_id in the URL for authentication and authorization, specify the same value.")] = None, scope : Annotated[Optional[StrictStr], Field(description="If you specified a value for scope in the URL for authentication and authorization, specify the same value.")] = None, brand_type : Annotated[Optional[StrictStr], Field(description="If you specified a value for brand_type in the URL for authentication and authorization, specify the same value.")] = None, **kwargs) -> AttachModuleResponse:  # noqa: E501
        ...

    @overload
    def attach_module(self, grant_type : Annotated[StrictStr, Field(..., description="authorization_code")], code : Annotated[StrictStr, Field(..., description="Authorization code received from the LINE Platform.")], redirect_uri : Annotated[StrictStr, Field(..., description="Specify the redirect_uri specified in the URL for authentication and authorization.")], code_verifier : Annotated[Optional[StrictStr], Field(description="Specify when using PKCE (Proof Key for Code Exchange) defined in the OAuth 2.0 extension specification as a countermeasure against authorization code interception attacks.")] = None, client_id : Annotated[Optional[StrictStr], Field(description="Instead of using Authorization header, you can use this parameter to specify the channel ID of the module channel. You can find the channel ID of the module channel in the LINE Developers Console. ")] = None, client_secret : Annotated[Optional[StrictStr], Field(description="Instead of using Authorization header, you can use this parameter to specify the channel secret of the module channel. You can find the channel secret of the module channel in the LINE Developers Console. ")] = None, region : Annotated[Optional[StrictStr], Field(description="If you specified a value for region in the URL for authentication and authorization, specify the same value. ")] = None, basic_search_id : Annotated[Optional[StrictStr], Field(description="If you specified a value for basic_search_id in the URL for authentication and authorization, specify the same value.")] = None, scope : Annotated[Optional[StrictStr], Field(description="If you specified a value for scope in the URL for authentication and authorization, specify the same value.")] = None, brand_type : Annotated[Optional[StrictStr], Field(description="If you specified a value for brand_type in the URL for authentication and authorization, specify the same value.")] = None, async_req: Optional[bool]=True, **kwargs) -> AttachModuleResponse:  # noqa: E501
        ...

    @validate_arguments
    def attach_module(self, grant_type : Annotated[StrictStr, Field(..., description="authorization_code")], code : Annotated[StrictStr, Field(..., description="Authorization code received from the LINE Platform.")], redirect_uri : Annotated[StrictStr, Field(..., description="Specify the redirect_uri specified in the URL for authentication and authorization.")], code_verifier : Annotated[Optional[StrictStr], Field(description="Specify when using PKCE (Proof Key for Code Exchange) defined in the OAuth 2.0 extension specification as a countermeasure against authorization code interception attacks.")] = None, client_id : Annotated[Optional[StrictStr], Field(description="Instead of using Authorization header, you can use this parameter to specify the channel ID of the module channel. You can find the channel ID of the module channel in the LINE Developers Console. ")] = None, client_secret : Annotated[Optional[StrictStr], Field(description="Instead of using Authorization header, you can use this parameter to specify the channel secret of the module channel. You can find the channel secret of the module channel in the LINE Developers Console. ")] = None, region : Annotated[Optional[StrictStr], Field(description="If you specified a value for region in the URL for authentication and authorization, specify the same value. ")] = None, basic_search_id : Annotated[Optional[StrictStr], Field(description="If you specified a value for basic_search_id in the URL for authentication and authorization, specify the same value.")] = None, scope : Annotated[Optional[StrictStr], Field(description="If you specified a value for scope in the URL for authentication and authorization, specify the same value.")] = None, brand_type : Annotated[Optional[StrictStr], Field(description="If you specified a value for brand_type in the URL for authentication and authorization, specify the same value.")] = None, async_req: Optional[bool]=None, **kwargs) -> Union[AttachModuleResponse, Awaitable[AttachModuleResponse]]:  # noqa: E501
        """attach_module  # noqa: E501

        Attach by operation of the module channel provider  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.attach_module(grant_type, code, redirect_uri, code_verifier, client_id, client_secret, region, basic_search_id, scope, brand_type, async_req=True)
        >>> result = thread.get()

        :param grant_type: authorization_code (required)
        :type grant_type: str
        :param code: Authorization code received from the LINE Platform. (required)
        :type code: str
        :param redirect_uri: Specify the redirect_uri specified in the URL for authentication and authorization. (required)
        :type redirect_uri: str
        :param code_verifier: Specify when using PKCE (Proof Key for Code Exchange) defined in the OAuth 2.0 extension specification as a countermeasure against authorization code interception attacks.
        :type code_verifier: str
        :param client_id: Instead of using Authorization header, you can use this parameter to specify the channel ID of the module channel. You can find the channel ID of the module channel in the LINE Developers Console. 
        :type client_id: str
        :param client_secret: Instead of using Authorization header, you can use this parameter to specify the channel secret of the module channel. You can find the channel secret of the module channel in the LINE Developers Console. 
        :type client_secret: str
        :param region: If you specified a value for region in the URL for authentication and authorization, specify the same value. 
        :type region: str
        :param basic_search_id: If you specified a value for basic_search_id in the URL for authentication and authorization, specify the same value.
        :type basic_search_id: str
        :param scope: If you specified a value for scope in the URL for authentication and authorization, specify the same value.
        :type scope: str
        :param brand_type: If you specified a value for brand_type in the URL for authentication and authorization, specify the same value.
        :type brand_type: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: AttachModuleResponse
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the attach_module_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        if async_req is not None:
            kwargs['async_req'] = async_req
        return self.attach_module_with_http_info(grant_type, code, redirect_uri, code_verifier, client_id, client_secret, region, basic_search_id, scope, brand_type, **kwargs)  # noqa: E501

    @validate_arguments
    def attach_module_with_http_info(self, grant_type : Annotated[StrictStr, Field(..., description="authorization_code")], code : Annotated[StrictStr, Field(..., description="Authorization code received from the LINE Platform.")], redirect_uri : Annotated[StrictStr, Field(..., description="Specify the redirect_uri specified in the URL for authentication and authorization.")], code_verifier : Annotated[Optional[StrictStr], Field(description="Specify when using PKCE (Proof Key for Code Exchange) defined in the OAuth 2.0 extension specification as a countermeasure against authorization code interception attacks.")] = None, client_id : Annotated[Optional[StrictStr], Field(description="Instead of using Authorization header, you can use this parameter to specify the channel ID of the module channel. You can find the channel ID of the module channel in the LINE Developers Console. ")] = None, client_secret : Annotated[Optional[StrictStr], Field(description="Instead of using Authorization header, you can use this parameter to specify the channel secret of the module channel. You can find the channel secret of the module channel in the LINE Developers Console. ")] = None, region : Annotated[Optional[StrictStr], Field(description="If you specified a value for region in the URL for authentication and authorization, specify the same value. ")] = None, basic_search_id : Annotated[Optional[StrictStr], Field(description="If you specified a value for basic_search_id in the URL for authentication and authorization, specify the same value.")] = None, scope : Annotated[Optional[StrictStr], Field(description="If you specified a value for scope in the URL for authentication and authorization, specify the same value.")] = None, brand_type : Annotated[Optional[StrictStr], Field(description="If you specified a value for brand_type in the URL for authentication and authorization, specify the same value.")] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """attach_module  # noqa: E501

        Attach by operation of the module channel provider  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.attach_module_with_http_info(grant_type, code, redirect_uri, code_verifier, client_id, client_secret, region, basic_search_id, scope, brand_type, async_req=True)
        >>> result = thread.get()

        :param grant_type: authorization_code (required)
        :type grant_type: str
        :param code: Authorization code received from the LINE Platform. (required)
        :type code: str
        :param redirect_uri: Specify the redirect_uri specified in the URL for authentication and authorization. (required)
        :type redirect_uri: str
        :param code_verifier: Specify when using PKCE (Proof Key for Code Exchange) defined in the OAuth 2.0 extension specification as a countermeasure against authorization code interception attacks.
        :type code_verifier: str
        :param client_id: Instead of using Authorization header, you can use this parameter to specify the channel ID of the module channel. You can find the channel ID of the module channel in the LINE Developers Console. 
        :type client_id: str
        :param client_secret: Instead of using Authorization header, you can use this parameter to specify the channel secret of the module channel. You can find the channel secret of the module channel in the LINE Developers Console. 
        :type client_secret: str
        :param region: If you specified a value for region in the URL for authentication and authorization, specify the same value. 
        :type region: str
        :param basic_search_id: If you specified a value for basic_search_id in the URL for authentication and authorization, specify the same value.
        :type basic_search_id: str
        :param scope: If you specified a value for scope in the URL for authentication and authorization, specify the same value.
        :type scope: str
        :param brand_type: If you specified a value for brand_type in the URL for authentication and authorization, specify the same value.
        :type brand_type: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(AttachModuleResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _host = self.line_base_path
        _params = locals()

        _all_params = [
            'grant_type',
            'code',
            'redirect_uri',
            'code_verifier',
            'client_id',
            'client_secret',
            'region',
            'basic_search_id',
            'scope',
            'brand_type'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method attach_module" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        if _params['grant_type']:
            _form_params.append(('grant_type', _params['grant_type']))

        if _params['code']:
            _form_params.append(('code', _params['code']))

        if _params['redirect_uri']:
            _form_params.append(('redirect_uri', _params['redirect_uri']))

        if _params['code_verifier']:
            _form_params.append(('code_verifier', _params['code_verifier']))

        if _params['client_id']:
            _form_params.append(('client_id', _params['client_id']))

        if _params['client_secret']:
            _form_params.append(('client_secret', _params['client_secret']))

        if _params['region']:
            _form_params.append(('region', _params['region']))

        if _params['basic_search_id']:
            _form_params.append(('basic_search_id', _params['basic_search_id']))

        if _params['scope']:
            _form_params.append(('scope', _params['scope']))

        if _params['brand_type']:
            _form_params.append(('brand_type', _params['brand_type']))

        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/x-www-form-urlencoded']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = ['basicAuth']  # noqa: E501

        _response_types_map = {
            '200': "AttachModuleResponse",
        }

        return self.api_client.call_api(
            '/module/auth/v1/token', 'POST',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            _host=_host,
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))



# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic.v1 import BaseModel, Field, StrictStr
from linebot.v3.messaging.models.imagemap_area import ImagemapArea
from linebot.v3.messaging.models.imagemap_external_link import ImagemapExternalLink

class ImagemapVideo(BaseModel):
    """
    ImagemapVideo
    """
    original_content_url: Optional[StrictStr] = Field(None, alias="originalContentUrl")
    preview_image_url: Optional[StrictStr] = Field(None, alias="previewImageUrl")
    area: Optional[ImagemapArea] = None
    external_link: Optional[ImagemapExternalLink] = Field(None, alias="externalLink")

    __properties = ["originalContentUrl", "previewImageUrl", "area", "externalLink"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ImagemapVideo:
        """Create an instance of ImagemapVideo from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic.v1 by calling `to_dict()` of area
        if self.area:
            _dict['area'] = self.area.to_dict()
        # override the default output from pydantic.v1 by calling `to_dict()` of external_link
        if self.external_link:
            _dict['externalLink'] = self.external_link.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> ImagemapVideo:
        """Create an instance of ImagemapVideo from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return ImagemapVideo.parse_obj(obj)

        _obj = ImagemapVideo.parse_obj({
            "original_content_url": obj.get("originalContentUrl"),
            "preview_image_url": obj.get("previewImageUrl"),
            "area": ImagemapArea.from_dict(obj.get("area")) if obj.get("area") is not None else None,
            "external_link": ImagemapExternalLink.from_dict(obj.get("externalLink")) if obj.get("externalLink") is not None else None
        })
        return _obj




# coding: utf-8

"""
    Webhook Type Definition

    Webhook event definition of the LINE Messaging API  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json



from pydantic.v1 import BaseModel, Field, StrictStr, validator

class LinkContent(BaseModel):
    """
    Content of the account link event.
    """
    result: StrictStr = Field(..., description="One of the following values to indicate whether linking the account was successful or not")
    nonce: StrictStr = Field(..., description="Specified nonce (number used once) when verifying the user ID.")

    __properties = ["result", "nonce"]

    @validator('result')
    def result_validate_enum(cls, value):
        """Validates the enum"""
        if value not in ('ok', 'failed'):
            raise ValueError("must be one of enum values ('ok', 'failed')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> LinkContent:
        """Create an instance of LinkContent from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> LinkContent:
        """Create an instance of LinkContent from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return LinkContent.parse_obj(obj)

        _obj = LinkContent.parse_obj({
            "result": obj.get("result"),
            "nonce": obj.get("nonce")
        })
        return _obj




# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the 'License'); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an 'AS IS' BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from __future__ import unicode_literals, absolute_import

import unittest

from linebot.models import TextMessage
from linebot.models.emojis import Emojis
from linebot.models.mentionee import Mentionee
from linebot.models.mention import Mention
from tests.models.serialize_test_case import SerializeTestCase


class TestTextMessage(SerializeTestCase):
    def test_emojis(self):
        arg = {
            "type": "text",
            "text": "$ LINE emoji $",
            'emojis': [
                Emojis(
                    index=0,
                    length=6,
                    product_id='5ac1bfd5040ab15980c9b435',
                    emoji_id='001'
                ),
                Emojis(
                    index=13,
                    length=6,
                    product_id='5ac1bfd5040ab15980c9b435',
                    emoji_id='002'
                ),
            ]
        }
        self.assertEqual(
            self.serialize_as_dict(arg, type=self.TEXT),
            TextMessage(**arg).as_json_dict()
        )

    def test_null_emojis(self):
        arg = {
            "type": "text",
            "text": "\uDBC0\uDC84 LINE original emoji"
        }
        self.assertEqual(
            self.serialize_as_dict(arg, type=self.TEXT),
            TextMessage(**arg).as_json_dict()
        )

    def test_mention(self):
        arg = {
            "type": "text",
            "text": "@example Hello, world! (love)",
            "mention": Mention(
                mentionees=[
                    Mentionee(
                        index=0,
                        length=8,
                        user_id="U850014438e..."
                    )
                ]
            ),
        }
        self.assertEqual(
            self.serialize_as_dict(arg, type=self.TEXT),
            TextMessage(**arg).as_json_dict(),
        )

    def test_null_mention(self):
        arg = {"type": "text", "text": "Hello, world!"}
        self.assertEqual(
            self.serialize_as_dict(arg, type=self.TEXT),
            TextMessage(**arg).as_json_dict(),
        )


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

"""linebot.models.imagemap module."""


from abc import ABCMeta

from future.utils import with_metaclass

from .base import Base
from .send_messages import SendMessage


class ImagemapSendMessage(SendMessage):
    """ImagemapSendMessage.

    https://developers.line.biz/en/reference/messaging-api/#imagemap-message

    Imagemaps are images with one or more links. You can assign one link for the entire image
    or multiple links which correspond to different regions of the image.
    """

    def __init__(self, base_url=None, alt_text=None, base_size=None,
                 video=None, actions=None, **kwargs):
        """__init__ method.

        :param str base_url: Base URL of image.
            HTTPS
        :param str alt_text: Alternative text
        :param base_size: Width and height of base image
        :type base_size: :py:class:`linebot.models.imagemap.BaseSize`
        :param video: Video in imagemap message
        :type video: :py:class:`linebot.models.imagemap.Video`
        :param actions: Action when tapped
        :type actions: list[T <= :py:class:`linebot.models.imagemap.ImagemapAction`]
        :param kwargs:
        """
        super(ImagemapSendMessage, self).__init__(**kwargs)

        self.type = 'imagemap'
        self.base_url = base_url
        self.alt_text = alt_text
        self.base_size = self.get_or_new_from_json_dict(
            base_size, BaseSize
        )
        self.video = self.get_or_new_from_json_dict(
            video, Video
        )

        new_actions = []
        if actions:
            for action in actions:
                action_obj = self.get_or_new_from_json_dict_with_types(
                    action, {
                        'uri': URIImagemapAction,
                        'message': MessageImagemapAction
                    }
                )
                if action_obj:
                    new_actions.append(action_obj)
        self.actions = new_actions


class BaseSize(Base):
    """BaseSize.

    https://developers.line.biz/en/reference/messaging-api/#imagemap-message
    """

    def __init__(self, width=None, height=None, **kwargs):
        """__init__ method.

        :param int width: Width of base image (set to 1040px）
        :param int height: Height of base image（set to the height
            that corresponds to a width of 1040px
        :param kwargs:
        """
        super(BaseSize, self).__init__(**kwargs)

        self.width = width
        self.height = height


class ImagemapAction(with_metaclass(ABCMeta, Base)):
    """ImagemapAction.

    https://developers.line.biz/en/reference/messaging-api/#imagemap-message
    """

    def __init__(self, **kwargs):
        """__init__ method.

        :param kwargs:
        """
        super(ImagemapAction, self).__init__(**kwargs)

        self.type = None


class URIImagemapAction(ImagemapAction):
    """URIImagemapAction.

    https://developers.line.biz/en/reference/messaging-api/#imagemap-message
    """

    def __init__(self, link_uri=None, area=None, **kwargs):
        """__init__ method.

        :param str link_uri: Webpage URL
        :param area: Defined tappable area
        :type area: :py:class:`linebot.models.imagemap.ImagemapArea`
        :param kwargs:
        """
        super(URIImagemapAction, self).__init__(**kwargs)

        self.type = 'uri'
        self.link_uri = link_uri
        self.area = self.get_or_new_from_json_dict(area, ImagemapArea)


class MessageImagemapAction(ImagemapAction):
    """MessageImagemapAction.

    https://developers.line.biz/en/reference/messaging-api/#imagemap-message
    """

    def __init__(self, text=None, area=None, **kwargs):
        """__init__ method.

        :param str text: Message to send
        :param area: Defined tappable area
        :type area: :py:class:`linebot.models.imagemap.ImagemapArea`
        :param kwargs:
        """
        super(MessageImagemapAction, self).__init__(**kwargs)

        self.type = 'message'
        self.text = text
        self.area = self.get_or_new_from_json_dict(area, ImagemapArea)


class ImagemapArea(Base):
    """ImagemapArea.

    https://developers.line.biz/en/reference/messaging-api/#imagemap-area-object

    Defines the size of the full imagemap with the width as 1040px.
    The top left is used as the origin of the area.
    """

    def __init__(self, x=None, y=None, width=None, height=None, **kwargs):
        """__init__ method.

        :param int x: Horizontal position of the tappable area
        :param int y: Vertical position of the tappable area
        :param int width: Width of the tappable area
        :param int height: Height of the tappable area
        :param kwargs:
        """
        super(ImagemapArea, self).__init__(**kwargs)

        self.x = x
        self.y = y
        self.width = width
        self.height = height


class Video(Base):
    """Video.

    https://developers.line.biz/en/reference/messaging-api/#imagemap-message

    Defines the properties of the video object in imagemap.
    """

    def __init__(self, original_content_url=None, preview_image_url=None,
                 area=None, external_link=None, **kwargs):
        """__init__ method.

        :param str original_content_url: URL of the video file
        :param str preview_image_url: URL of the preview image
        :param area: Defined video area
        :type area: :py:class:`linebot.models.imagemap.ImagemapArea`
        :param external_link: Defined video external link
        :type external_link: :py:class:`linebot.models.imagemap.ExternalLink`
        :param kwargs:
        """
        super(Video, self).__init__(**kwargs)

        self.original_content_url = original_content_url
        self.preview_image_url = preview_image_url
        self.area = self.get_or_new_from_json_dict(area, ImagemapArea)
        self.external_link = self.get_or_new_from_json_dict(external_link, ExternalLink)


class ExternalLink(Base):
    """ExternalLink.

    https://developers.line.biz/en/reference/messaging-api/#imagemap-message

    Defines URL and label of external link in video.
    """

    def __init__(self, link_uri=None, label=None, **kwargs):
        """__init__ method.

        :param str link_uri: Webpage URL
        :param str label: Label
        :param kwargs:
        """
        super(ExternalLink, self).__init__(**kwargs)

        self.link_uri = link_uri
        self.label = label



# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic.v1 import Field, StrictBool, StrictInt, StrictStr, validator
from linebot.v3.messaging.models.action import Action
from linebot.v3.messaging.models.flex_component import FlexComponent

class FlexImage(FlexComponent):
    """
    FlexImage
    https://developers.line.biz/en/reference/messaging-api/#f-image
    """
    url: StrictStr = Field(..., description="Image URL (Max character limit: 2000) Protocol: HTTPS (TLS 1.2 or later) Image format: JPEG or PNG Maximum image size: 1024×1024 pixels Maximum file size: 10 MB (300 KB when the animated property is true) ")
    flex: Optional[StrictInt] = Field(None, description="The ratio of the width or height of this component within the parent box.")
    margin: Optional[StrictStr] = Field(None, description="The minimum amount of space to include before this component in its parent container. ")
    position: Optional[StrictStr] = Field(None, description="Reference for offsetTop, offsetBottom, offsetStart, and offsetEnd. Specify one of the following values:  `relative`: Use the previous box as reference. `absolute`: Use the top left of parent element as reference. The default value is relative. ")
    offset_top: Optional[StrictStr] = Field(None, alias="offsetTop", description="Offset.")
    offset_bottom: Optional[StrictStr] = Field(None, alias="offsetBottom", description="Offset.")
    offset_start: Optional[StrictStr] = Field(None, alias="offsetStart", description="Offset.")
    offset_end: Optional[StrictStr] = Field(None, alias="offsetEnd", description="Offset.")
    align: Optional[StrictStr] = Field(None, description="Alignment style in horizontal direction. ")
    gravity: Optional[StrictStr] = Field(None, description="Alignment style in vertical direction.")
    size: Optional[StrictStr] = Field('md', description="The maximum image width. This is md by default. ")
    aspect_ratio: Optional[StrictStr] = Field(None, alias="aspectRatio", description="Aspect ratio of the image. `{width}:{height}` format. Specify the value of `{width}` and `{height}` in the range from `1` to `100000`. However, you cannot set `{height}` to a value that is more than three times the value of `{width}`. The default value is `1:1`. ")
    aspect_mode: Optional[StrictStr] = Field(None, alias="aspectMode", description="The display style of the image if the aspect ratio of the image and that specified by the aspectRatio property do not match. ")
    background_color: Optional[StrictStr] = Field(None, alias="backgroundColor", description="Background color of the image. Use a hexadecimal color code.")
    action: Optional[Action] = None
    animated: Optional[StrictBool] = Field(False, description="When this is `true`, an animated image (APNG) plays. You can specify a value of true up to 10 images in a single message. You can't send messages that exceed this limit. This is `false` by default. Animated images larger than 300 KB aren't played back. ")
    type: str = "image"

    __properties = ["type", "url", "flex", "margin", "position", "offsetTop", "offsetBottom", "offsetStart", "offsetEnd", "align", "gravity", "size", "aspectRatio", "aspectMode", "backgroundColor", "action", "animated"]

    @validator('position')
    def position_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('relative', 'absolute'):
            raise ValueError("must be one of enum values ('relative', 'absolute')")
        return value

    @validator('align')
    def align_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('start', 'end', 'center'):
            raise ValueError("must be one of enum values ('start', 'end', 'center')")
        return value

    @validator('gravity')
    def gravity_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('top', 'bottom', 'center'):
            raise ValueError("must be one of enum values ('top', 'bottom', 'center')")
        return value

    @validator('aspect_mode')
    def aspect_mode_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('fit', 'cover'):
            raise ValueError("must be one of enum values ('fit', 'cover')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> FlexImage:
        """Create an instance of FlexImage from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic.v1 by calling `to_dict()` of action
        if self.action:
            _dict['action'] = self.action.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> FlexImage:
        """Create an instance of FlexImage from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return FlexImage.parse_obj(obj)

        _obj = FlexImage.parse_obj({
            "type": obj.get("type"),
            "url": obj.get("url"),
            "flex": obj.get("flex"),
            "margin": obj.get("margin"),
            "position": obj.get("position"),
            "offset_top": obj.get("offsetTop"),
            "offset_bottom": obj.get("offsetBottom"),
            "offset_start": obj.get("offsetStart"),
            "offset_end": obj.get("offsetEnd"),
            "align": obj.get("align"),
            "gravity": obj.get("gravity"),
            "size": obj.get("size") if obj.get("size") is not None else 'md',
            "aspect_ratio": obj.get("aspectRatio"),
            "aspect_mode": obj.get("aspectMode"),
            "background_color": obj.get("backgroundColor"),
            "action": Action.from_dict(obj.get("action")) if obj.get("action") is not None else None,
            "animated": obj.get("animated") if obj.get("animated") is not None else False
        })
        return _obj




# coding: utf-8

"""
    LIFF server API

    LIFF Server API.  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class LiffScope(str, Enum):
    """
    Array of scopes required for some LIFF SDK methods to function. The default value is `[\"profile\", \"chat_message.write\"]`.  
    """

    """
    allowed enum values
    """
    OPENID = 'openid'
    EMAIL = 'email'
    PROFILE = 'profile'
    CHAT_MESSAGE_DOT_WRITE = 'chat_message.write'

    @classmethod
    def from_json(cls, json_str: str) -> LiffScope:
        """Create an instance of LiffScope from a JSON string"""
        return LiffScope(json.loads(json_str))





# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class AudienceGroupCreateRoute(str, Enum):
    """
    How the audience was created. One of:  - `OA_MANAGER`: Audience created with [LINE Official Account Manager](https://manager.line.biz/). - `MESSAGING_API`: Audience created with Messaging API. - `POINT_AD`: Audience created with [LINE Points Ads](https://www.linebiz.com/jp/service/line-point-ad/) (Japanese only). - `AD_MANAGER`: Audience created with [LINE Ads](https://admanager.line.biz/). 
    """

    """
    allowed enum values
    """
    OA_MANAGER = 'OA_MANAGER'
    MESSAGING_API = 'MESSAGING_API'
    POINT_AD = 'POINT_AD'
    AD_MANAGER = 'AD_MANAGER'

    @classmethod
    def from_json(cls, json_str: str) -> AudienceGroupCreateRoute:
        """Create an instance of AudienceGroupCreateRoute from a JSON string"""
        return AudienceGroupCreateRoute(json.loads(json_str))





# coding: utf-8

# flake8: noqa
"""
    Channel Access Token API

    This document describes Channel Access Token API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


# import models into model package
from linebot.v3.oauth.models.channel_access_token_key_ids_response import ChannelAccessTokenKeyIdsResponse
from linebot.v3.oauth.models.error_response import ErrorResponse
from linebot.v3.oauth.models.issue_channel_access_token_response import IssueChannelAccessTokenResponse
from linebot.v3.oauth.models.issue_short_lived_channel_access_token_response import IssueShortLivedChannelAccessTokenResponse
from linebot.v3.oauth.models.issue_stateless_channel_access_token_response import IssueStatelessChannelAccessTokenResponse
from linebot.v3.oauth.models.verify_channel_access_token_response import VerifyChannelAccessTokenResponse



# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from __future__ import unicode_literals, absolute_import

import unittest

from linebot import (
    LineBotApi
)


class TestLineBotApi(unittest.TestCase):
    def setUp(self):
        self.tested = LineBotApi('channel_secret')

    def test_get_content(self):
        pass
        # message_content = self.tested.get_content(12345)
        # with open('/tmp/content.jpg', 'wb') as f:
        #     for chunk in message_content.iter_content():
        #         f.write(chunk)
        #     f.close()


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

"""linebot.exceptions module."""

from abc import ABCMeta

from future.utils import with_metaclass

from deprecated import deprecated

from .deprecations import (
    LineBotSdkDeprecatedIn30
)


class BaseError(with_metaclass(ABCMeta, Exception)):
    """Base Exception class."""

    def __init__(self, message='-'):
        """__init__ method.

        :param str message: Human readable message
        """
        self.message = message

    def __repr__(self):
        """repr."""
        return str(self)

    def __str__(self):
        """str.

        :rtype: str
        """
        return '<{0} [{1}]>'.format(
            self.__class__.__name__, self.message)


@deprecated(reason="Use 'from linebot.v3.exceptions import InvalidSignatureError' and v3 webhook handlers instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
class InvalidSignatureError(BaseError):
    """When Webhook signature does NOT match, this error will be raised."""

    def __init__(self, message='-'):
        """__init__ method.

        :param str message: Human readable message
        """
        super(InvalidSignatureError, self).__init__(message)


class LineBotApiError(BaseError):
    """When LINE Messaging API response error, this error will be raised."""

    def __init__(
            self,
            status_code,
            headers,
            request_id=None,
            accepted_request_id=None,
            error=None
    ):
        """__init__ method.

        :param int status_code: HTTP status code
        :param headers: Response headers
        :type headers: dict[str, str]
        :param str request_id: (optional) Request ID. A unique ID is generated for each request
        :param str accepted_request_id: (optional) The same request has already been accepted
        :param error: (optional) Error class object.
        :type error: :py:class:`linebot.models.error.Error`
        """
        super(LineBotApiError, self).__init__(error.message)

        self.status_code = status_code
        self.headers = headers
        self.request_id = request_id
        self.accepted_request_id = accepted_request_id
        self.error = error

    def __str__(self):
        """str.

        :rtype: str
        """
        if self.accepted_request_id:
            return "{0}: status_code={1}, request_id={2}, " \
                   "accepted_request_id={3}, error_response={4}, headers={5}" \
                .format(self.__class__.__name__,
                        self.status_code,
                        self.request_id,
                        self.accepted_request_id,
                        self.error,
                        self.headers)
        return '{0}: status_code={1}, request_id={2}, error_response={3}, headers={4}'.format(
            self.__class__.__name__, self.status_code, self.request_id, self.error,
            self.headers)



# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

"""linebot.models.sources module."""


import warnings
from abc import ABCMeta, abstractproperty

from future.utils import with_metaclass

from .base import Base

from deprecated import deprecated

from linebot.deprecations import (
    LineBotSdkDeprecatedIn30
)


@deprecated(reason="Use 'from linebot.v3.webhooks import Source' instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
class Source(with_metaclass(ABCMeta, Base)):
    """Abstract Base Class of Source."""

    def __init__(self, **kwargs):
        """__init__ method.

        :param kwargs:
        """
        super(Source, self).__init__(**kwargs)
        self.type = None

    @abstractproperty
    def sender_id(self):
        """Abstract property of id to send a message.

        If SourceUser, return user_id.
        If SourceGroup, return group_id.
        If SourceRoom, return room_id.

        'sender_id' is deprecated.

        :rtype: str
        """
        warnings.warn("'sender_id' is deprecated.", DeprecationWarning, stacklevel=2)
        raise NotImplementedError


@deprecated(reason="Use 'from linebot.v3.webhooks import UserSource' instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
class SourceUser(Source):
    """SourceUser.

    https://developers.line.biz/en/reference/messaging-api/#source-user

    JSON object which contains the source user of the event.
    """

    def __init__(self, user_id=None, **kwargs):
        """__init__ method.

        :param str user_id: ID of the source user
        :param kwargs:
        """
        super(SourceUser, self).__init__(**kwargs)

        self.type = 'user'
        self.user_id = user_id

    @property
    def sender_id(self):
        """Alias of user_id.

        'sender_id' is deprecated. Use 'user_id' instead.

        :rtype: str
        """
        warnings.warn("'sender_id' is deprecated.", DeprecationWarning, stacklevel=2)
        return self.user_id


@deprecated(reason="Use 'from linebot.v3.webhooks import GroupSource' instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
class SourceGroup(Source):
    """SourceGroup.

    https://developers.line.biz/en/reference/messaging-api/#source-group

    JSON object which contains the source group of the event.
    """

    def __init__(self, group_id=None, user_id=None, **kwargs):
        """__init__ method.

        :param str group_id: ID of the source group
        :param str user_id: ID of the source user
        :param kwargs:
        """
        super(SourceGroup, self).__init__(**kwargs)

        self.type = 'group'
        self.group_id = group_id
        self.user_id = user_id

    @property
    def sender_id(self):
        """Alias of group_id.

        'sender_id' is deprecated. Use 'group_id' instead.

        :rtype: str
        """
        warnings.warn("'sender_id' is deprecated.", DeprecationWarning, stacklevel=2)
        return self.group_id


@deprecated(reason="Use 'from linebot.v3.webhooks import RoomSource' instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
class SourceRoom(Source):
    """SourceRoom.

    https://developers.line.biz/en/reference/messaging-api/#source-room

    JSON object which contains the source room of the event.
    """

    def __init__(self, room_id=None, user_id=None, **kwargs):
        """__init__ method.

        :param str room_id: ID of the source room
        :param str user_id: ID of the source user
        :param kwargs:
        """
        super(SourceRoom, self).__init__(**kwargs)

        self.type = 'room'
        self.room_id = room_id
        self.user_id = user_id

    @property
    def sender_id(self):
        """Alias of room_id.

        'sender_id' is deprecated. Use 'room_id' instead.

        :rtype: str
        """
        warnings.warn("'sender_id' is deprecated.", DeprecationWarning, stacklevel=2)
        return self.room_id



# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from typing import Optional
from pydantic.v1 import BaseModel, Field, StrictInt, StrictStr, validator

class NarrowcastProgressResponse(BaseModel):
    """
    NarrowcastProgressResponse
    https://developers.line.biz/en/reference/messaging-api/#get-narrowcast-progress-status
    """
    phase: StrictStr = Field(..., description="The current status. One of:  `waiting`: Messages are not yet ready to be sent. They are currently being filtered or processed in some way. `sending`: Messages are currently being sent. `succeeded`: Messages were sent successfully. This may not mean the messages were successfully received. `failed`: Messages failed to be sent. Use the failedDescription property to find the cause of the failure. ")
    success_count: Optional[StrictInt] = Field(None, alias="successCount", description="The number of users who successfully received the message.")
    failure_count: Optional[StrictInt] = Field(None, alias="failureCount", description="The number of users who failed to send the message.")
    target_count: Optional[StrictInt] = Field(None, alias="targetCount", description="The number of intended recipients of the message.")
    failed_description: Optional[StrictStr] = Field(None, alias="failedDescription", description="The reason the message failed to be sent. This is only included with a `phase` property value of `failed`.")
    error_code: Optional[StrictInt] = Field(None, alias="errorCode", description="Error summary. This is only included with a phase property value of failed. One of:  `1`: An internal error occurred. `2`: An error occurred because there weren't enough recipients. `3`: A conflict error of requests occurs because a request that has already been accepted is retried. `4`: An audience of less than 50 recipients is included as a condition of sending. ")
    accepted_time: datetime = Field(..., alias="acceptedTime", description="Narrowcast message request accepted time in milliseconds.  Format: ISO 8601 (e.g. 2020-12-03T10:15:30.121Z) Timezone: UTC ")
    completed_time: Optional[datetime] = Field(None, alias="completedTime", description="Processing of narrowcast message request completion time in milliseconds. Returned when the phase property is succeeded or failed.  Format: ISO 8601 (e.g. 2020-12-03T10:15:30.121Z) Timezone: UTC ")

    __properties = ["phase", "successCount", "failureCount", "targetCount", "failedDescription", "errorCode", "acceptedTime", "completedTime"]

    @validator('phase')
    def phase_validate_enum(cls, value):
        """Validates the enum"""
        if value not in ('waiting', 'sending', 'succeeded', 'failed'):
            raise ValueError("must be one of enum values ('waiting', 'sending', 'succeeded', 'failed')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> NarrowcastProgressResponse:
        """Create an instance of NarrowcastProgressResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> NarrowcastProgressResponse:
        """Create an instance of NarrowcastProgressResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return NarrowcastProgressResponse.parse_obj(obj)

        _obj = NarrowcastProgressResponse.parse_obj({
            "phase": obj.get("phase"),
            "success_count": obj.get("successCount"),
            "failure_count": obj.get("failureCount"),
            "target_count": obj.get("targetCount"),
            "failed_description": obj.get("failedDescription"),
            "error_code": obj.get("errorCode"),
            "accepted_time": obj.get("acceptedTime"),
            "completed_time": obj.get("completedTime")
        })
        return _obj




# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

"""linebot.async_http_client module."""

from __future__ import unicode_literals

from abc import ABCMeta, abstractmethod, abstractproperty

from future.utils import with_metaclass


class AsyncHttpClient(with_metaclass(ABCMeta)):
    """Abstract Base Classes of HttpClient."""

    DEFAULT_TIMEOUT = 5

    def __init__(self, timeout=DEFAULT_TIMEOUT):
        """__init__ method.

        :param timeout: (optional) How long to wait for the server
            to send data before giving up, as a float,
            or a (connect timeout, read timeout) float tuple.
            Default is :py:attr:`DEFAULT_TIMEOUT`
        :type timeout: float | tuple(float, float)
        :rtype: T <= :py:class:`HttpResponse`
        :return: HttpResponse instance
        """
        self.timeout = timeout

    @abstractmethod
    async def get(self, url, headers=None, params=None, timeout=None):
        """GET request.

        :param str url: Request url
        :param dict headers: (optional) Request headers
        :param dict params: (optional) Request query parameter
        :param timeout: (optional), How long to wait for the server
            to send data before giving up, as a float,
            or a (connect timeout, read timeout) float tuple.
            Default is :py:attr:`self.timeout`
        :type timeout: float | tuple(float, float)
        :rtype: T <= :py:class:`HttpResponse`
        :return: HttpResponse instance
        """
        raise NotImplementedError

    @abstractmethod
    async def post(self, url, headers=None, data=None, timeout=None):
        """POST request.

        :param str url: Request url
        :param dict headers: (optional) Request headers
        :param data: (optional) Dictionary, bytes, or file-like object to send in the body
        :param timeout: (optional), How long to wait for the server
            to send data before giving up, as a float,
            or a (connect timeout, read timeout) float tuple.
            Default is :py:attr:`self.timeout`
        :type timeout: float | tuple(float, float)
        :rtype: T <= :py:class:`HttpResponse`
        :return: HttpResponse instance
        """
        raise NotImplementedError

    @abstractmethod
    async def delete(self, url, headers=None, data=None, timeout=None):
        """DELETE request.

        :param str url: Request url
        :param dict headers: (optional) Request headers
        :param data: (optional) Dictionary, bytes, or file-like object to send in the body
        :param timeout: (optional), How long to wait for the server
            to send data before giving up, as a float,
            or a (connect timeout, read timeout) float tuple.
            Default is :py:attr:`self.timeout`
        :type timeout: float | tuple(float, float)
        :rtype: T <= :py:class:`HttpResponse`
        :return: HttpResponse instance
        """
        raise NotImplementedError

    @abstractmethod
    async def put(self, url, headers=None, data=None, timeout=None):
        """PUT request.

        :param str url: Request url
        :param dict headers: (optional) Request headers
        :param data: (optional) Dictionary, bytes, or file-like object to send in the body
        :param timeout: (optional), How long to wait for the server
            to send data before giving up, as a float,
            or a (connect timeout, read timeout) float tuple.
            Default is :py:attr:`self.timeout`
        :type timeout: float | tuple(float, float)
        :rtype: :py:class:`AsyncHttpResponse`
        :return: AsyncHttpResponse instance
        """
        raise NotImplementedError


class AsyncHttpResponse(with_metaclass(ABCMeta)):
    """HttpResponse."""

    @abstractproperty
    def status_code(self):
        """Get status code."""
        raise NotImplementedError

    @abstractproperty
    def headers(self):
        """Get headers."""
        raise NotImplementedError

    @abstractproperty
    async def text(self):
        """Get response body as text-decoded."""
        raise NotImplementedError

    @abstractproperty
    async def content(self):
        """Get response body as binary."""
        raise NotImplementedError

    @abstractproperty
    def json(self):
        """Get response body as json-decoded."""
        raise NotImplementedError

    @abstractmethod
    def iter_content(self, chunk_size=1024):
        """Get response body as iterator content (stream).

        :param int chunk_size:
        """
        raise NotImplementedError



# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the 'License'); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an 'AS IS' BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from __future__ import unicode_literals, absolute_import

import unittest

from linebot.models import (
    TextSendMessage,
    StickerSendMessage,
    VideoSendMessage,
    AudioSendMessage,
    LocationSendMessage,
    ImageSendMessage,
    QuickReplyButton,
    QuickReply,
    CameraRollAction,
    CameraAction
)
from tests.models.serialize_test_case import SerializeTestCase


class TestSendMessages(SerializeTestCase):
    def test_text_message(self):
        arg = {
            'text': 'Hello, world'
        }
        self.assertEqual(
            self.serialize_as_dict(arg, type=self.TEXT),
            TextSendMessage(**arg).as_json_dict()
        )

    def test_sticker_message(self):
        arg = {
            'package_id': '1',
            'sticker_id': '1'
        }
        self.assertEqual(
            self.serialize_as_dict(arg, type=self.STICKER),
            StickerSendMessage(**arg).as_json_dict()
        )

    def test_image_message(self):
        arg = {
            'original_content_url': 'https://example.com/original.jpg',
            'preview_image_url': 'https://example.com/preview.jpg'
        }
        self.assertEqual(
            self.serialize_as_dict(arg, type=self.IMAGE),
            ImageSendMessage(**arg).as_json_dict()
        )

    def test_video_message(self):
        arg = {
            'type': 'video',
            'original_content_url': 'https://example.com/original.mp4',
            'preview_image_url': 'https://example.com/preview.jpg'
        }
        self.assertEqual(
            self.serialize_as_dict(arg, type=self.VIDEO),
            VideoSendMessage(**arg).as_json_dict()
        )

    def test_video_message_wtih_track(self):
        arg = {
            'type': 'video',
            'original_content_url': 'https://example.com/original.mp4',
            'preview_image_url': 'https://example.com/preview.jpg',
            "tracking_id": "track_id"
        }
        self.assertEqual(
            self.serialize_as_dict(arg, type=self.VIDEO),
            VideoSendMessage(**arg).as_json_dict()
        )

    def test_audio_message(self):
        arg = {
            'original_content_url': 'https://example.com/original.m4a',
            'duration': 60000
        }
        self.assertEqual(
            self.serialize_as_dict(arg, type=self.AUDIO),
            AudioSendMessage(**arg).as_json_dict()
        )

    def test_location_message(self):
        arg = {
            'title': 'my location',
            'address': '〒150-0002 東京都渋谷区渋谷２丁目２１−１',
            'latitude': 35.65910807942215,
            'longitude': 139.70372892916203
        }
        self.assertEqual(
            self.serialize_as_dict(arg, type=self.LOCATION),
            LocationSendMessage(**arg).as_json_dict()
        )

    def test_quick_reply_button(self):
        arg = {
            'action': CameraRollAction(label='Send photo')
        }
        self.assertEqual(
            self.serialize_as_dict(arg, type=self.ACTION),
            QuickReplyButton(**arg).as_json_dict()
        )

    def test_quick_reply(self):
        arg = {
            'items': [
                QuickReplyButton(action=CameraRollAction(label='Send photo')),
                QuickReplyButton(action=CameraAction(label='Open camera')),
            ]
        }
        self.assertEqual(
            self.serialize_as_dict(arg),
            QuickReply(**arg).as_json_dict()
        )


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic.v1 import Field, StrictInt, StrictStr, conlist, validator
from linebot.v3.messaging.models.action import Action
from linebot.v3.messaging.models.flex_box_background import FlexBoxBackground
from linebot.v3.messaging.models.flex_component import FlexComponent

class FlexBox(FlexComponent):
    """
    FlexBox
    """
    layout: StrictStr = Field(...)
    flex: Optional[StrictInt] = None
    contents: conlist(FlexComponent) = Field(...)
    spacing: Optional[StrictStr] = None
    margin: Optional[StrictStr] = None
    position: Optional[StrictStr] = None
    offset_top: Optional[StrictStr] = Field(None, alias="offsetTop")
    offset_bottom: Optional[StrictStr] = Field(None, alias="offsetBottom")
    offset_start: Optional[StrictStr] = Field(None, alias="offsetStart")
    offset_end: Optional[StrictStr] = Field(None, alias="offsetEnd")
    background_color: Optional[StrictStr] = Field(None, alias="backgroundColor")
    border_color: Optional[StrictStr] = Field(None, alias="borderColor")
    border_width: Optional[StrictStr] = Field(None, alias="borderWidth")
    corner_radius: Optional[StrictStr] = Field(None, alias="cornerRadius")
    width: Optional[StrictStr] = None
    max_width: Optional[StrictStr] = Field(None, alias="maxWidth")
    height: Optional[StrictStr] = None
    max_height: Optional[StrictStr] = Field(None, alias="maxHeight")
    padding_all: Optional[StrictStr] = Field(None, alias="paddingAll")
    padding_top: Optional[StrictStr] = Field(None, alias="paddingTop")
    padding_bottom: Optional[StrictStr] = Field(None, alias="paddingBottom")
    padding_start: Optional[StrictStr] = Field(None, alias="paddingStart")
    padding_end: Optional[StrictStr] = Field(None, alias="paddingEnd")
    action: Optional[Action] = None
    justify_content: Optional[StrictStr] = Field(None, alias="justifyContent")
    align_items: Optional[StrictStr] = Field(None, alias="alignItems")
    background: Optional[FlexBoxBackground] = None
    type: str = "box"

    __properties = ["type", "layout", "flex", "contents", "spacing", "margin", "position", "offsetTop", "offsetBottom", "offsetStart", "offsetEnd", "backgroundColor", "borderColor", "borderWidth", "cornerRadius", "width", "maxWidth", "height", "maxHeight", "paddingAll", "paddingTop", "paddingBottom", "paddingStart", "paddingEnd", "action", "justifyContent", "alignItems", "background"]

    @validator('layout')
    def layout_validate_enum(cls, value):
        """Validates the enum"""
        if value not in ('horizontal', 'vertical', 'baseline'):
            raise ValueError("must be one of enum values ('horizontal', 'vertical', 'baseline')")
        return value

    @validator('position')
    def position_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('relative', 'absolute'):
            raise ValueError("must be one of enum values ('relative', 'absolute')")
        return value

    @validator('justify_content')
    def justify_content_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('center', 'flex-start', 'flex-end', 'space-between', 'space-around', 'space-evenly'):
            raise ValueError("must be one of enum values ('center', 'flex-start', 'flex-end', 'space-between', 'space-around', 'space-evenly')")
        return value

    @validator('align_items')
    def align_items_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('center', 'flex-start', 'flex-end'):
            raise ValueError("must be one of enum values ('center', 'flex-start', 'flex-end')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> FlexBox:
        """Create an instance of FlexBox from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic.v1 by calling `to_dict()` of each item in contents (list)
        _items = []
        if self.contents:
            for _item in self.contents:
                if _item:
                    _items.append(_item.to_dict())
            _dict['contents'] = _items
        # override the default output from pydantic.v1 by calling `to_dict()` of action
        if self.action:
            _dict['action'] = self.action.to_dict()
        # override the default output from pydantic.v1 by calling `to_dict()` of background
        if self.background:
            _dict['background'] = self.background.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> FlexBox:
        """Create an instance of FlexBox from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return FlexBox.parse_obj(obj)

        _obj = FlexBox.parse_obj({
            "type": obj.get("type"),
            "layout": obj.get("layout"),
            "flex": obj.get("flex"),
            "contents": [FlexComponent.from_dict(_item) for _item in obj.get("contents")] if obj.get("contents") is not None else None,
            "spacing": obj.get("spacing"),
            "margin": obj.get("margin"),
            "position": obj.get("position"),
            "offset_top": obj.get("offsetTop"),
            "offset_bottom": obj.get("offsetBottom"),
            "offset_start": obj.get("offsetStart"),
            "offset_end": obj.get("offsetEnd"),
            "background_color": obj.get("backgroundColor"),
            "border_color": obj.get("borderColor"),
            "border_width": obj.get("borderWidth"),
            "corner_radius": obj.get("cornerRadius"),
            "width": obj.get("width"),
            "max_width": obj.get("maxWidth"),
            "height": obj.get("height"),
            "max_height": obj.get("maxHeight"),
            "padding_all": obj.get("paddingAll"),
            "padding_top": obj.get("paddingTop"),
            "padding_bottom": obj.get("paddingBottom"),
            "padding_start": obj.get("paddingStart"),
            "padding_end": obj.get("paddingEnd"),
            "action": Action.from_dict(obj.get("action")) if obj.get("action") is not None else None,
            "justify_content": obj.get("justifyContent"),
            "align_items": obj.get("alignItems"),
            "background": FlexBoxBackground.from_dict(obj.get("background")) if obj.get("background") is not None else None
        })
        return _obj




# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from __future__ import unicode_literals, absolute_import

import json
import unittest

import responses

from linebot import (
    LineBotApi
)
from linebot.models import (
    ImagemapSendMessage, BaseSize, URIImagemapAction,
    ImagemapArea, MessageImagemapAction,
    Video, ExternalLink
)


class TestLineBotApi(unittest.TestCase):
    def setUp(self):
        self.tested = LineBotApi('channel_secret')

        self.imagemap_message = ImagemapSendMessage(
            base_url='https://example.com/base',
            alt_text='this is an imagemap',
            base_size=BaseSize(height=1040, width=1040),
            video=Video(
                original_content_url='https://example.com/video.mp4',
                preview_image_url='https://example.com/video_preview.jpg',
                area=ImagemapArea(
                    x=0, y=0, width=1040, height=585
                ),
                external_link=ExternalLink(
                    link_uri='https://example.com/see_more.html',
                    label='See More',
                ),
            ),
            actions=[
                URIImagemapAction(
                    link_uri='https://example.com/',
                    area=ImagemapArea(
                        x=0, y=0, width=520, height=1040
                    )
                ),
                MessageImagemapAction(
                    text='hello',
                    area=ImagemapArea(
                        x=520, y=0, width=520, height=1040
                    )
                )
            ]
        )

        self.message = [{
            "type": "imagemap",
            "baseUrl": "https://example.com/base",
            "altText": "this is an imagemap",
            "baseSize": {
                "height": 1040,
                "width": 1040
            },
            "video": {
                "originalContentUrl": "https://example.com/video.mp4",
                "previewImageUrl": "https://example.com/video_preview.jpg",
                "area": {
                    "x": 0,
                    "y": 0,
                    "width": 1040,
                    "height": 585
                },
                "externalLink": {
                    "linkUri": "https://example.com/see_more.html",
                    "label": "See More"
                }
            },
            "actions": [
                {
                    "type": "uri",
                    "linkUri": "https://example.com/",
                    "area": {
                        "x": 0,
                        "y": 0,
                        "width": 520,
                        "height": 1040
                    }
                },
                {
                    "type": "message",
                    "text": "hello",
                    "area": {
                        "x": 520,
                        "y": 0,
                        "width": 520,
                        "height": 1040
                    }
                }
            ]
        }]

    @responses.activate
    def test_push_imagemap_message(self):
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/push',
            json={}, status=200
        )

        self.tested.push_message('to', self.imagemap_message)

        request = responses.calls[0].request
        self.assertEqual(request.method, 'POST')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/push')
        self.assertEqual(
            json.loads(request.body),
            {
                "to": "to",
                'notificationDisabled': False,
                "messages": self.message
            }
        )

    @responses.activate
    def test_reply_imagemap_message(self):
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/reply',
            json={}, status=200
        )

        self.tested.reply_message('replyToken', self.imagemap_message)

        request = responses.calls[0].request
        self.assertEqual(request.method, 'POST')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/reply')
        self.assertEqual(
            json.loads(request.body),
            {
                "replyToken": "replyToken",
                'notificationDisabled': False,
                "messages": self.message
            }
        )

    @responses.activate
    def test_multicast_imagemap_message(self):
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/multicast',
            json={}, status=200
        )

        self.tested.multicast(['to1', 'to2'], self.imagemap_message)

        request = responses.calls[0].request
        self.assertEqual(request.method, 'POST')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/multicast')
        self.assertEqual(
            json.loads(request.body),
            {
                "to": ['to1', 'to2'],
                'notificationDisabled': False,
                "messages": self.message
            }
        )


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from typing import Optional
from pydantic.v1 import BaseModel, Field
from linebot.v3.messaging.models.rich_menu_batch_progress_phase import RichMenuBatchProgressPhase

class RichMenuBatchProgressResponse(BaseModel):
    """
    RichMenuBatchProgressResponse
    https://developers.line.biz/en/reference/messaging-api/#get-batch-control-rich-menus-progress-status-response
    """
    phase: RichMenuBatchProgressPhase = Field(...)
    accepted_time: datetime = Field(..., alias="acceptedTime", description="The accepted time in milliseconds of the request of batch control the rich menu.  Format: ISO 8601 (e.g. 2023-06-08T10:15:30.121Z) Timezone: UTC ")
    completed_time: Optional[datetime] = Field(None, alias="completedTime", description="The completed time in milliseconds of rich menu batch control. Returned when the phase property is succeeded or failed.  Format: ISO 8601 (e.g. 2023-06-08T10:15:30.121Z) Timezone: UTC ")

    __properties = ["phase", "acceptedTime", "completedTime"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> RichMenuBatchProgressResponse:
        """Create an instance of RichMenuBatchProgressResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> RichMenuBatchProgressResponse:
        """Create an instance of RichMenuBatchProgressResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return RichMenuBatchProgressResponse.parse_obj(obj)

        _obj = RichMenuBatchProgressResponse.parse_obj({
            "phase": obj.get("phase"),
            "accepted_time": obj.get("acceptedTime"),
            "completed_time": obj.get("completedTime")
        })
        return _obj




# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from __future__ import unicode_literals, absolute_import

import json
import unittest

from linebot.models import Base


class Hoge(Base):
    def __init__(self, title=None, content=None, hoge_bar=None, **kwargs):
        super(Hoge, self).__init__(**kwargs)

        self.title = title
        self.content = content
        self.hoge_bar = hoge_bar


class TestBase(unittest.TestCase):
    def test_as_json_string(self):
        self.assertEqual(
            Hoge().as_json_string(),
            '{}')
        self.assertEqual(
            Hoge(title='title').as_json_string(),
            '{"title": "title"}')
        self.assertEqual(
            Hoge(title='title', content='content').as_json_string(),
            '{"content": "content", "title": "title"}')
        self.assertEqual(
            Hoge(title='title', content={"hoge": "hoge"}).as_json_string(),
            '{"content": {"hoge": "hoge"}, "title": "title"}')
        self.assertEqual(
            Hoge(title=[1, 2]).as_json_string(),
            '{"title": [1, 2]}')
        self.assertEqual(
            Hoge(hoge_bar='hoge_bar').as_json_string(),
            '{"hogeBar": "hoge_bar"}')

    def test_as_json_dict(self):
        self.assertEqual(
            Hoge().as_json_dict(),
            {})
        self.assertEqual(
            Hoge(title='title').as_json_dict(),
            {'title': 'title'})
        self.assertEqual(
            Hoge(title='title', content='content').as_json_dict(),
            {'content': 'content', 'title': 'title'})
        self.assertEqual(
            Hoge(title='title', content={"hoge": "hoge"}).as_json_dict(),
            {'content': {'hoge': 'hoge'}, 'title': 'title'})
        self.assertEqual(
            Hoge(title=[1, 2]).as_json_dict(),
            {'title': [1, 2]})

    def test_new_from_json_dict(self):
        self.assertEqual(
            Hoge.new_from_json_dict({"title": "title"}),
            Hoge(title='title'))
        self.assertEqual(
            Hoge.new_from_json_dict(json.loads('{"title": "title"}')),
            Hoge(title='title'))
        self.assertEqual(
            Hoge.new_from_json_dict({"hoge_bar": "hoge_bar"}),
            Hoge(hoge_bar='hoge_bar'))
        self.assertEqual(
            Hoge.new_from_json_dict({"hogeBar": "hoge_bar"}),
            Hoge(hoge_bar='hoge_bar'))


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic.v1 import Field, StrictBool, StrictInt, StrictStr, conlist, validator
from linebot.v3.messaging.models.action import Action
from linebot.v3.messaging.models.flex_component import FlexComponent
from linebot.v3.messaging.models.flex_span import FlexSpan

class FlexText(FlexComponent):
    """
    FlexText
    """
    flex: Optional[StrictInt] = None
    text: Optional[StrictStr] = None
    size: Optional[StrictStr] = None
    align: Optional[StrictStr] = None
    gravity: Optional[StrictStr] = None
    color: Optional[StrictStr] = None
    weight: Optional[StrictStr] = None
    style: Optional[StrictStr] = None
    decoration: Optional[StrictStr] = None
    wrap: Optional[StrictBool] = None
    line_spacing: Optional[StrictStr] = Field(None, alias="lineSpacing")
    margin: Optional[StrictStr] = None
    position: Optional[StrictStr] = None
    offset_top: Optional[StrictStr] = Field(None, alias="offsetTop")
    offset_bottom: Optional[StrictStr] = Field(None, alias="offsetBottom")
    offset_start: Optional[StrictStr] = Field(None, alias="offsetStart")
    offset_end: Optional[StrictStr] = Field(None, alias="offsetEnd")
    action: Optional[Action] = None
    max_lines: Optional[StrictInt] = Field(None, alias="maxLines")
    contents: Optional[conlist(FlexSpan)] = None
    adjust_mode: Optional[StrictStr] = Field(None, alias="adjustMode")
    scaling: Optional[StrictBool] = None
    type: str = "text"

    __properties = ["type", "flex", "text", "size", "align", "gravity", "color", "weight", "style", "decoration", "wrap", "lineSpacing", "margin", "position", "offsetTop", "offsetBottom", "offsetStart", "offsetEnd", "action", "maxLines", "contents", "adjustMode", "scaling"]

    @validator('align')
    def align_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('start', 'end', 'center'):
            raise ValueError("must be one of enum values ('start', 'end', 'center')")
        return value

    @validator('gravity')
    def gravity_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('top', 'bottom', 'center'):
            raise ValueError("must be one of enum values ('top', 'bottom', 'center')")
        return value

    @validator('weight')
    def weight_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('regular', 'bold'):
            raise ValueError("must be one of enum values ('regular', 'bold')")
        return value

    @validator('style')
    def style_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('normal', 'italic'):
            raise ValueError("must be one of enum values ('normal', 'italic')")
        return value

    @validator('decoration')
    def decoration_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('none', 'underline', 'line-through'):
            raise ValueError("must be one of enum values ('none', 'underline', 'line-through')")
        return value

    @validator('position')
    def position_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('relative', 'absolute'):
            raise ValueError("must be one of enum values ('relative', 'absolute')")
        return value

    @validator('adjust_mode')
    def adjust_mode_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('shrink-to-fit'):
            raise ValueError("must be one of enum values ('shrink-to-fit')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> FlexText:
        """Create an instance of FlexText from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic.v1 by calling `to_dict()` of action
        if self.action:
            _dict['action'] = self.action.to_dict()
        # override the default output from pydantic.v1 by calling `to_dict()` of each item in contents (list)
        _items = []
        if self.contents:
            for _item in self.contents:
                if _item:
                    _items.append(_item.to_dict())
            _dict['contents'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> FlexText:
        """Create an instance of FlexText from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return FlexText.parse_obj(obj)

        _obj = FlexText.parse_obj({
            "type": obj.get("type"),
            "flex": obj.get("flex"),
            "text": obj.get("text"),
            "size": obj.get("size"),
            "align": obj.get("align"),
            "gravity": obj.get("gravity"),
            "color": obj.get("color"),
            "weight": obj.get("weight"),
            "style": obj.get("style"),
            "decoration": obj.get("decoration"),
            "wrap": obj.get("wrap"),
            "line_spacing": obj.get("lineSpacing"),
            "margin": obj.get("margin"),
            "position": obj.get("position"),
            "offset_top": obj.get("offsetTop"),
            "offset_bottom": obj.get("offsetBottom"),
            "offset_start": obj.get("offsetStart"),
            "offset_end": obj.get("offsetEnd"),
            "action": Action.from_dict(obj.get("action")) if obj.get("action") is not None else None,
            "max_lines": obj.get("maxLines"),
            "contents": [FlexSpan.from_dict(_item) for _item in obj.get("contents")] if obj.get("contents") is not None else None,
            "adjust_mode": obj.get("adjustMode"),
            "scaling": obj.get("scaling")
        })
        return _obj




# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

"""linebot.aiohttp_async_http_client module."""

from linebot import AsyncHttpClient, AsyncHttpResponse


class AiohttpAsyncHttpClient(AsyncHttpClient):
    """HttpClient implemented by requests."""

    def __init__(self, session, timeout=AsyncHttpClient.DEFAULT_TIMEOUT):
        """__init__ method.

        :param timeout: (optional) How long to wait for the server
            to send data before giving up, as a float,
            or a (connect timeout, read timeout) float tuple.
            Default is :py:attr:`DEFAULT_TIMEOUT`
        :type timeout: float | tuple(float, float)
        """
        super(AiohttpAsyncHttpClient, self).__init__(timeout)
        self.session = session

    async def get(self, url, headers=None, params=None, timeout=None):
        """GET request.

        :param str url: Request url
        :param dict headers: (optional) Request headers
        :param dict params: (optional) Request query parameter
        :param timeout: (optional), How long to wait for the server
            to send data before giving up, as a float,
            or a (connect timeout, read timeout) float tuple.
            Default is :py:attr:`self.timeout`
        :type timeout: float | tuple(float, float)
        :rtype: :py:class:`AsyncHttpResponse`
        :return: AsyncHttpResponse instance
        """
        if timeout is None:
            timeout = self.timeout

        response = await self.session.get(url, headers=headers, params=params, timeout=timeout)
        return AiohttpAsyncHttpResponse(response)

    async def post(self, url, headers=None, data=None, timeout=None):
        """POST request.

        :param str url: Request url
        :param dict headers: (optional) Request headers
        :param data: (optional) Dictionary, bytes, or file-like object to send in the body
        :param timeout: (optional), How long to wait for the server
            to send data before giving up, as a float,
            or a (connect timeout, read timeout) float tuple.
            Default is :py:attr:`self.timeout`
        :type timeout: float | tuple(float, float)
        :rtype: :py:class:`AiohttpAsyncHttpResponse`
        :return: AiohttpAsyncHttpResponse instance
        """
        if timeout is None:
            timeout = self.timeout

        response = await self.session.post(url, headers=headers, data=data, timeout=timeout)
        return AiohttpAsyncHttpResponse(response)

    async def delete(self, url, headers=None, data=None, timeout=None):
        """DELETE request.

        :param str url: Request url
        :param dict headers: (optional) Request headers
        :param data: (optional) Dictionary, bytes, or file-like object to send in the body
        :param timeout: (optional), How long to wait for the server
            to send data before giving up, as a float,
            or a (connect timeout, read timeout) float tuple.
            Default is :py:attr:`self.timeout`
        :type timeout: float | tuple(float, float)
        :rtype: :py:class:`AiohttpAsyncHttpResponse`
        :return: AiohttpAsyncHttpResponse instance
        """
        if timeout is None:
            timeout = self.timeout

        response = await self.session.delete(url, headers=headers, data=data, timeout=timeout)
        return AiohttpAsyncHttpResponse(response)

    async def put(self, url, headers=None, data=None, timeout=None):
        """PUT request.

        :param str url: Request url
        :param dict headers: (optional) Request headers
        :param data: (optional) Dictionary, bytes, or file-like object to send in the body
        :param timeout: (optional), How long to wait for the server
            to send data before giving up, as a float,
            or a (connect timeout, read timeout) float tuple.
            Default is :py:attr:`self.timeout`
        :type timeout: float | tuple(float, float)
        :rtype: :py:class:`AiohttpAsyncHttpResponse`
        :return: AiohttpAsyncHttpResponse instance
        """
        if timeout is None:
            timeout = self.timeout

        response = await self.session.put(url, headers=headers, data=data, timeout=timeout)
        return AiohttpAsyncHttpResponse(response)


class AiohttpAsyncHttpResponse(AsyncHttpResponse):
    """AsyncHttpResponse implemented by aiohttp's response."""

    def __init__(self, response):
        """__init__ method.

        :param response: aiohttp's response
        """
        self.response = response

    @property
    def status_code(self):
        """Get status code."""
        return self.response.status

    @property
    def headers(self):
        """Get headers.

        :rtype :py:class:`aiohttp.multidict.CIMultiDictProxy`
        """
        return self.response.headers

    @property
    async def text(self):
        """Get response body as text-decoded."""
        return await self.response.text()

    @property
    async def content(self):
        """Get response body as binary."""
        return await self.response.content.read()

    @property
    async def json(self):
        """Get response body as json-decoded."""
        return await self.response.json()

    def iter_content(self, chunk_size=1024):
        """Get response body as iterator content (stream).

        :param int chunk_size:
        """
        return self.response.content.iter_chunked(chunk_size)



# flake8: noqa

# import apis into api package
from linebot.v3.oauth.api.channel_access_token import ChannelAccessToken


# Async version
from linebot.v3.oauth.api.async_channel_access_token import AsyncChannelAccessToken




# coding: utf-8

# flake8: noqa

"""
    Channel Access Token API

    This document describes Channel Access Token API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


# import apis into sdk package
from linebot.v3.oauth.api.channel_access_token import ChannelAccessToken

from linebot.v3.oauth.api.async_channel_access_token import AsyncChannelAccessToken


# import ApiClient
from linebot.v3.oauth.api_response import ApiResponse
from linebot.v3.oauth.api_client import ApiClient
from linebot.v3.oauth.async_api_client import AsyncApiClient
from linebot.v3.oauth.configuration import Configuration
from linebot.v3.oauth.exceptions import OpenApiException
from linebot.v3.oauth.exceptions import ApiTypeError
from linebot.v3.oauth.exceptions import ApiValueError
from linebot.v3.oauth.exceptions import ApiKeyError
from linebot.v3.oauth.exceptions import ApiAttributeError
from linebot.v3.oauth.exceptions import ApiException

# import models into sdk package
from linebot.v3.oauth.models.channel_access_token_key_ids_response import ChannelAccessTokenKeyIdsResponse
from linebot.v3.oauth.models.error_response import ErrorResponse
from linebot.v3.oauth.models.issue_channel_access_token_response import IssueChannelAccessTokenResponse
from linebot.v3.oauth.models.issue_short_lived_channel_access_token_response import IssueShortLivedChannelAccessTokenResponse
from linebot.v3.oauth.models.issue_stateless_channel_access_token_response import IssueStatelessChannelAccessTokenResponse
from linebot.v3.oauth.models.verify_channel_access_token_response import VerifyChannelAccessTokenResponse



# coding: utf-8

"""
    LIFF server API

    LIFF Server API.  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic.v1 import BaseModel, Field, StrictStr, conlist
from linebot.v3.liff.models.liff_bot_prompt import LiffBotPrompt
from linebot.v3.liff.models.liff_features import LiffFeatures
from linebot.v3.liff.models.liff_scope import LiffScope
from linebot.v3.liff.models.liff_view import LiffView

class AddLiffAppRequest(BaseModel):
    """
    AddLiffAppRequest
    https://developers.line.biz/en/reference/liff-server/#add-liff-app
    """
    view: LiffView = Field(...)
    description: Optional[StrictStr] = Field(None, description="Name of the LIFF app.  The LIFF app name can't include \"LINE\" or similar strings, or inappropriate strings. ")
    features: Optional[LiffFeatures] = None
    permanent_link_pattern: Optional[StrictStr] = Field(None, alias="permanentLinkPattern", description="How additional information in LIFF URLs is handled. Specify `concat`. ")
    scope: Optional[conlist(LiffScope)] = None
    bot_prompt: Optional[LiffBotPrompt] = Field(None, alias="botPrompt")

    __properties = ["view", "description", "features", "permanentLinkPattern", "scope", "botPrompt"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> AddLiffAppRequest:
        """Create an instance of AddLiffAppRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic.v1 by calling `to_dict()` of view
        if self.view:
            _dict['view'] = self.view.to_dict()
        # override the default output from pydantic.v1 by calling `to_dict()` of features
        if self.features:
            _dict['features'] = self.features.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> AddLiffAppRequest:
        """Create an instance of AddLiffAppRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return AddLiffAppRequest.parse_obj(obj)

        _obj = AddLiffAppRequest.parse_obj({
            "view": LiffView.from_dict(obj.get("view")) if obj.get("view") is not None else None,
            "description": obj.get("description"),
            "features": LiffFeatures.from_dict(obj.get("features")) if obj.get("features") is not None else None,
            "permanent_link_pattern": obj.get("permanentLinkPattern"),
            "scope": obj.get("scope"),
            "bot_prompt": obj.get("botPrompt")
        })
        return _obj




# coding: utf-8

"""
    Webhook Type Definition

    Webhook event definition of the LINE Messaging API  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json




from linebot.v3.webhooks.models.delivery_context import DeliveryContext
from linebot.v3.webhooks.models.event import Event
from linebot.v3.webhooks.models.event_mode import EventMode
from linebot.v3.webhooks.models.source import Source

class BotResumedEvent(Event):
    """
    This event indicates that the LINE Official Account has returned from the suspended state. Sent to the webhook URL server of the module channel.
    """
    type: str = "botResumed"

    __properties = ["type", "source", "timestamp", "mode", "webhookEventId", "deliveryContext"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> BotResumedEvent:
        """Create an instance of BotResumedEvent from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic.v1 by calling `to_dict()` of source
        if self.source:
            _dict['source'] = self.source.to_dict()
        # override the default output from pydantic.v1 by calling `to_dict()` of delivery_context
        if self.delivery_context:
            _dict['deliveryContext'] = self.delivery_context.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> BotResumedEvent:
        """Create an instance of BotResumedEvent from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return BotResumedEvent.parse_obj(obj)

        _obj = BotResumedEvent.parse_obj({
            "type": obj.get("type"),
            "source": Source.from_dict(obj.get("source")) if obj.get("source") is not None else None,
            "timestamp": obj.get("timestamp"),
            "mode": obj.get("mode"),
            "webhook_event_id": obj.get("webhookEventId"),
            "delivery_context": DeliveryContext.from_dict(obj.get("deliveryContext")) if obj.get("deliveryContext") is not None else None
        })
        return _obj




# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from __future__ import unicode_literals, absolute_import

import unittest

from linebot.exceptions import LineBotApiError
from linebot.models import Error, ErrorDetail


class TestExceptions(unittest.TestCase):
    maxDiff = None

    def test_str(self):
        headers = {'X-Line-Request-Id': 'f70dd685-499a-4231-a441-f24b8d4fba21'}
        line_bot_api_error = LineBotApiError(
            status_code=400,
            request_id='f70dd685-499a-4231-a441-f24b8d4fba21',
            headers=headers,
            error=Error(message='The request body has 1 error(s)',
                        details=[ErrorDetail(message='May not be empty',
                                             property='messages[0].text')]))
        self.assertEqual(
            line_bot_api_error.__str__(),
            'LineBotApiError: status_code=400, request_id=f70dd685-499a-4231-a441-f24b8d4fba21, '
            'error_response={"details": [{"message": "May not be empty", '
            '"property": "messages[0].text"}], "message": "The request body has 1 error(s)"}, '
            + 'headers={}'.format(headers)
        )

    def test_accepted_str(self):
        headers = {
            'X-Line-Request-Id': '123e4567-e89b-12d3-a456-426655440002',
            'X-Line-Accepted-Request-Id': '123e4567-e89b-12d3-a456-426655440001'
        }
        line_bot_api_error = LineBotApiError(
            status_code=409,
            request_id='123e4567-e89b-12d3-a456-426655440002',
            accepted_request_id='123e4567-e89b-12d3-a456-426655440001',
            headers=headers,
            error=Error(message='The retry key is already accepted')
        )
        self.assertEqual(
            line_bot_api_error.__str__(),
            'LineBotApiError: status_code=409, request_id=123e4567-e89b-12d3-a456-426655440002, '
            'accepted_request_id=123e4567-e89b-12d3-a456-426655440001, '
            'error_response={"details": [], "message": "The retry key is already accepted"}, '
            + 'headers={}'.format(headers)
        )


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic.v1 import BaseModel, Field, conlist, constr, validator
from linebot.v3.messaging.models.rich_menu_batch_operation import RichMenuBatchOperation

class RichMenuBatchRequest(BaseModel):
    """
    RichMenuBatchRequest
    """
    operations: conlist(RichMenuBatchOperation, max_items=1000) = Field(..., description="Array of Rich menu operation object...")
    resume_request_key: Optional[constr(strict=True, max_length=100, min_length=1)] = Field(None, alias="resumeRequestKey", description="Key for retry. Key value is a string matching the regular expression pattern")

    __properties = ["operations", "resumeRequestKey"]

    @validator('resume_request_key')
    def resume_request_key_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not re.match(r"^[a-zA-Z0-9_-]{1,100}$", value):
            raise ValueError(r"must validate the regular expression /^[a-zA-Z0-9_-]{1,100}$/")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> RichMenuBatchRequest:
        """Create an instance of RichMenuBatchRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic.v1 by calling `to_dict()` of each item in operations (list)
        _items = []
        if self.operations:
            for _item in self.operations:
                if _item:
                    _items.append(_item.to_dict())
            _dict['operations'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> RichMenuBatchRequest:
        """Create an instance of RichMenuBatchRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return RichMenuBatchRequest.parse_obj(obj)

        _obj = RichMenuBatchRequest.parse_obj({
            "operations": [RichMenuBatchOperation.from_dict(_item) for _item in obj.get("operations")] if obj.get("operations") is not None else None,
            "resume_request_key": obj.get("resumeRequestKey")
        })
        return _obj




# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic.v1 import Field, StrictInt, StrictStr, validator
from linebot.v3.messaging.models.discount_price_info_response import DiscountPriceInfoResponse

class DiscountExplicitPriceInfoResponse(DiscountPriceInfoResponse):
    """
    DiscountExplicitPriceInfoResponse
    """
    currency: Optional[StrictStr] = Field(None, description="Currency code (e.g., JPY, THB, TWD).")
    price_after_discount: Optional[StrictInt] = Field(None, alias="priceAfterDiscount")
    original_price: Optional[StrictInt] = Field(None, alias="originalPrice")
    type: str = "explicit"

    __properties = ["type", "currency", "priceAfterDiscount", "originalPrice"]

    @validator('currency')
    def currency_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('JPY', 'THB', 'TWD'):
            raise ValueError("must be one of enum values ('JPY', 'THB', 'TWD')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> DiscountExplicitPriceInfoResponse:
        """Create an instance of DiscountExplicitPriceInfoResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> DiscountExplicitPriceInfoResponse:
        """Create an instance of DiscountExplicitPriceInfoResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return DiscountExplicitPriceInfoResponse.parse_obj(obj)

        _obj = DiscountExplicitPriceInfoResponse.parse_obj({
            "type": obj.get("type"),
            "currency": obj.get("currency"),
            "price_after_discount": obj.get("priceAfterDiscount"),
            "original_price": obj.get("originalPrice")
        })
        return _obj




# coding: utf-8

# flake8: noqa
"""
    LIFF server API

    LIFF Server API.  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


# import models into model package
from linebot.v3.liff.models.add_liff_app_request import AddLiffAppRequest
from linebot.v3.liff.models.add_liff_app_response import AddLiffAppResponse
from linebot.v3.liff.models.get_all_liff_apps_response import GetAllLiffAppsResponse
from linebot.v3.liff.models.liff_app import LiffApp
from linebot.v3.liff.models.liff_bot_prompt import LiffBotPrompt
from linebot.v3.liff.models.liff_features import LiffFeatures
from linebot.v3.liff.models.liff_scope import LiffScope
from linebot.v3.liff.models.liff_view import LiffView
from linebot.v3.liff.models.update_liff_app_request import UpdateLiffAppRequest
from linebot.v3.liff.models.update_liff_view import UpdateLiffView



# coding: utf-8

"""
    Channel Access Token API

    This document describes Channel Access Token API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic.v1 import BaseModel, Field, StrictInt, StrictStr

class VerifyChannelAccessTokenResponse(BaseModel):
    """
    Verification result
    """
    client_id: StrictStr = Field(..., description="The channel ID for which the channel access token was issued.")
    expires_in: StrictInt = Field(..., description="Number of seconds before the channel access token expires.")
    scope: Optional[StrictStr] = Field(None, description="Permissions granted to the channel access token.")

    __properties = ["client_id", "expires_in", "scope"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> VerifyChannelAccessTokenResponse:
        """Create an instance of VerifyChannelAccessTokenResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> VerifyChannelAccessTokenResponse:
        """Create an instance of VerifyChannelAccessTokenResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return VerifyChannelAccessTokenResponse.parse_obj(obj)

        _obj = VerifyChannelAccessTokenResponse.parse_obj({
            "client_id": obj.get("client_id"),
            "expires_in": obj.get("expires_in"),
            "scope": obj.get("scope")
        })
        return _obj




# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from __future__ import unicode_literals, absolute_import

import json
import unittest

import responses

from linebot import (
    LineBotApi
)
from linebot.models import (
    TemplateSendMessage, ButtonsTemplate,
    PostbackAction, MessageAction,
    URIAction, AltUri, DatetimePickerAction,
    ConfirmTemplate, CarouselTemplate, CarouselColumn,
    ImageCarouselTemplate, ImageCarouselColumn
)


class TestLineBotApi(unittest.TestCase):
    maxDiff = None

    def setUp(self):
        self.tested = LineBotApi('channel_secret')

        self.button_template_message = TemplateSendMessage(
            alt_text='Buttons template',
            template=ButtonsTemplate(
                thumbnail_image_url='https://example.com/image.jpg',
                title='Menu', text='Please select',
                actions=[
                    PostbackAction(
                        label='postback', display_text='postback text',
                        data='action=buy&itemid=1'
                    ),
                    MessageAction(
                        label='message', text='message text'
                    ),
                    URIAction(
                        label='uri', uri='http://example.com/',
                        alt_uri=AltUri(desktop="http://example.com/desktop")
                    )
                ]
            )
        )

        self.button_message = [{
            "type": "template",
            "altText": "Buttons template",
            "template": {
                "type": "buttons",
                "thumbnailImageUrl":
                    "https://example.com/image.jpg",
                "title": "Menu",
                "text": "Please select",
                "actions": [
                    {
                        "type": "postback",
                        "label": "postback",
                        "displayText": "postback text",
                        "data": "action=buy&itemid=1"
                    },
                    {
                        "type": "message",
                        "label": "message",
                        "text": "message text"
                    },
                    {
                        "type": "uri",
                        "label": "uri",
                        "uri": "http://example.com/",
                        "altUri": {
                            "desktop": "http://example.com/desktop"
                        }
                    }
                ]
            }
        }]

        self.confirm_template_message = TemplateSendMessage(
            alt_text='Confirm template',
            template=ConfirmTemplate(
                text='Are you sure?',
                actions=[
                    PostbackAction(
                        label='postback', display_text='postback text',
                        data='action=buy&itemid=1'
                    ),
                    MessageAction(
                        label='message', text='message text'
                    )
                ]
            )
        )

        self.confirm_message = [{
            "type": "template",
            "altText": "Confirm template",
            "template": {
                "type": "confirm",
                "text": "Are you sure?",
                "actions": [
                    {
                        "type": "postback",
                        "label": "postback",
                        "displayText": "postback text",
                        "data": "action=buy&itemid=1"
                    },
                    {
                        "type": "message",
                        "label": "message",
                        "text": "message text"
                    }
                ]
            }
        }]

        self.carousel_template_message = TemplateSendMessage(
            alt_text='Carousel template',
            template=CarouselTemplate(
                columns=[
                    CarouselColumn(
                        thumbnail_image_url='https://example.com'
                                            '/item1.jpg',
                        title='this is menu1', text='description1',
                        actions=[
                            PostbackAction(
                                label='postback1', display_text='postback text1',
                                data='action=buy&itemid=1'
                            ),
                            MessageAction(
                                label='message1', text='message text1'
                            ),
                            URIAction(
                                label='uri1',
                                uri='http://example.com/1'
                            )
                        ]
                    ),
                    CarouselColumn(
                        thumbnail_image_url='https://example.com'
                                            '/item2.jpg',
                        image_background_color='#000000',
                        title='this is menu2', text='description2',
                        actions=[
                            PostbackAction(
                                label='postback2', display_text='postback text2',
                                data='action=buy&itemid=2'
                            ),
                            MessageAction(
                                label='message2', text='message text2'
                            ),
                            URIAction(
                                label='uri2',
                                uri='http://example.com/2'
                            )
                        ]
                    ),
                    CarouselColumn(
                        thumbnail_image_url='https://example.com'
                                            '/item3.jpg',
                        title='this is menu3', text='description3',
                        actions=[
                            DatetimePickerAction(
                                label="datetime picker date",
                                data="action=sell&itemid=2&mode=date",
                                mode="date",
                                initial="2013-04-01",
                                min="2011-06-23",
                                max="2017-09-08"
                            ),
                            DatetimePickerAction(
                                label="datetime picker time",
                                data="action=sell&itemid=2&mode=time",
                                mode="time",
                                initial="10:00",
                                min="00:00",
                                max="23:59"
                            ),
                            DatetimePickerAction(
                                label="datetime picker datetime",
                                data="action=sell&itemid=2&mode=datetime",
                                mode="datetime",
                                initial="2013-04-01T10:00",
                                min="2011-06-23T00:00",
                                max="2017-09-08T23:59"
                            )
                        ]
                    )
                ]
            )
        )

        self.carousel_message = [{
            "type": "template",
            "altText": "Carousel template",
            "template": {
                "type": "carousel",
                "columns": [
                    {
                        "thumbnailImageUrl":
                            "https://example.com/item1.jpg",
                        "title": "this is menu1",
                        "text": "description1",
                        "actions": [
                            {
                                "type": "postback",
                                "label": "postback1",
                                "displayText": "postback text1",
                                "data": "action=buy&itemid=1"
                            },
                            {
                                "type": "message",
                                "label": "message1",
                                "text": "message text1"
                            },
                            {
                                "type": "uri",
                                "label": "uri1",
                                "uri": "http://example.com/1"
                            }
                        ]
                    },
                    {
                        "thumbnailImageUrl":
                            "https://example.com/item2.jpg",
                        "imageBackgroundColor": "#000000",
                        "title": "this is menu2",
                        "text": "description2",
                        "actions": [
                            {
                                "type": "postback",
                                "label": "postback2",
                                "displayText": "postback text2",
                                "data": "action=buy&itemid=2"
                            },
                            {
                                "type": "message",
                                "label": "message2",
                                "text": "message text2"
                            },
                            {
                                "type": "uri",
                                "label": "uri2",
                                "uri": "http://example.com/2"
                            }
                        ]
                    },
                    {
                        "thumbnailImageUrl":
                            "https://example.com/item3.jpg",
                        "title": "this is menu3",
                        "text": "description3",
                        "actions": [
                            {
                                "type": "datetimepicker",
                                "label": "datetime picker date",
                                "data": "action=sell&itemid=2&mode=date",
                                "mode": "date",
                                "initial": "2013-04-01",
                                "min": "2011-06-23",
                                "max": "2017-09-08"
                            },
                            {
                                "type": "datetimepicker",
                                "label": "datetime picker time",
                                "data": "action=sell&itemid=2&mode=time",
                                "mode": "time",
                                "initial": "10:00",
                                "min": "00:00",
                                "max": "23:59"
                            },
                            {
                                "type": "datetimepicker",
                                "label": "datetime picker datetime",
                                "data": "action=sell&itemid=2&mode=datetime",
                                "mode": "datetime",
                                "initial": "2013-04-01T10:00",
                                "min": "2011-06-23T00:00",
                                "max": "2017-09-08T23:59"
                            }
                        ]
                    }
                ],
            }
        }]

        self.image_carousel_template_message = TemplateSendMessage(
            alt_text='Image carousel template',
            template=ImageCarouselTemplate(
                columns=[
                    ImageCarouselColumn(
                        image_url='https://example.com/'
                                  'item1.jpg',
                        action=PostbackAction(
                            label='postback1',
                            data='action=buy&itemid=1'
                        )
                    ),
                    ImageCarouselColumn(
                        image_url='https://example.com'
                                  '/item2.jpg',
                        action=MessageAction(
                            label='message2',
                            text='message text2'
                        )
                    ),
                    ImageCarouselColumn(
                        image_url='https://example.com/'
                                  'item3.jpg',
                        action=URIAction(
                            label='uri1',
                            uri='https://example.com/1'
                        )
                    )
                ]
            )
        )

        self.image_carousel_message = [{
            "type": "template",
            "altText": "Image carousel template",
            "template": {
                "type": "image_carousel",
                "columns": [
                    {
                        "imageUrl": "https://example.com/item1.jpg",
                        "action": {
                            "type": "postback",
                            "label": "postback1",
                            "data": "action=buy&itemid=1",
                        }
                    },
                    {
                        "imageUrl": "https://example.com/item2.jpg",
                        "action": {
                            "type": "message",
                            "label": "message2",
                            "text": "message text2"
                        }
                    },
                    {
                        "imageUrl": "https://example.com/item3.jpg",
                        "action": {
                            "type": "uri",
                            "label": "uri1",
                            "uri": "https://example.com/1"
                        }
                    }
                ]
            }
        }]

    @responses.activate
    def test_push_buttons_template_message(self):
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/push',
            json={}, status=200
        )

        self.tested.push_message('to', self.button_template_message)

        request = responses.calls[0].request
        self.assertEqual(request.method, 'POST')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/push')
        self.assertEqual(
            json.loads(request.body),
            {
                "to": "to",
                'notificationDisabled': False,
                "messages": self.button_message
            }
        )

    @responses.activate
    def test_reply_buttons_template_message(self):
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/reply',
            json={}, status=200
        )

        self.tested.reply_message('replyToken', self.button_template_message)

        request = responses.calls[0].request
        self.assertEqual(request.method, 'POST')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/reply')
        self.assertEqual(
            json.loads(request.body),
            {
                "replyToken": "replyToken",
                'notificationDisabled': False,
                "messages": self.button_message
            }
        )

    @responses.activate
    def test_push_confirm_template_message(self):
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/push',
            json={}, status=200
        )

        self.tested.push_message('to', self.confirm_template_message)

        request = responses.calls[0].request
        self.assertEqual(request.method, 'POST')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/push')
        self.assertEqual(
            json.loads(request.body),
            {
                "to": "to",
                'notificationDisabled': False,
                "messages": self.confirm_message
            }
        )

    @responses.activate
    def test_reply_confirm_template_message(self):
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/reply',
            json={}, status=200
        )

        self.tested.reply_message('replyToken', self.confirm_template_message)

        request = responses.calls[0].request
        self.assertEqual(request.method, 'POST')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/reply'
        )
        self.assertEqual(
            json.loads(request.body),
            {
                "replyToken": "replyToken",
                'notificationDisabled': False,
                "messages": self.confirm_message
            }
        )

    @responses.activate
    def test_push_carousel_template_message(self):
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/push',
            json={}, status=200
        )

        self.tested.push_message('to', self.carousel_template_message)

        request = responses.calls[0].request
        self.assertEqual(request.method, 'POST')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/push')
        self.assertEqual(
            json.loads(request.body),
            {
                "to": "to",
                'notificationDisabled': False,
                "messages": self.carousel_message
            }
        )

    @responses.activate
    def test_reply_carousel_template_message(self):
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/reply',
            json={}, status=200
        )

        self.tested.reply_message('replyToken', self.carousel_template_message)

        request = responses.calls[0].request
        self.assertEqual(request.method, 'POST')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/reply')
        self.assertEqual(
            json.loads(request.body),
            {
                "replyToken": "replyToken",
                'notificationDisabled': False,
                "messages": self.carousel_message
            }
        )

    @responses.activate
    def test_multicast_carousel_template_message(self):
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/multicast',
            json={}, status=200
        )

        self.tested.multicast(['to1', 'to2'], self.carousel_template_message)

        request = responses.calls[0].request
        self.assertEqual(request.method, 'POST')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/multicast')
        self.assertEqual(
            json.loads(request.body),
            {
                "to": ['to1', 'to2'],
                'notificationDisabled': False,
                "messages": self.carousel_message
            }
        )

    @responses.activate
    def test_push_image_carousel_template_message(self):
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/push',
            json={}, status=200
        )

        self.tested.push_message('to', self.image_carousel_template_message)

        request = responses.calls[0].request
        self.assertEqual(request.method, 'POST')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/push')
        self.assertEqual(
            json.loads(request.body),
            {
                "to": "to",
                'notificationDisabled': False,
                "messages": self.image_carousel_message
            }
        )

    @responses.activate
    def test_reply_image_carousel_template_message(self):
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/reply',
            json={}, status=200
        )

        self.tested.reply_message('replyToken', self.image_carousel_template_message)

        request = responses.calls[0].request
        self.assertEqual(request.method, 'POST')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/reply')
        self.assertEqual(
            json.loads(request.body),
            {
                "replyToken": "replyToken",
                'notificationDisabled': False,
                "messages": self.image_carousel_message
            }
        )

    @responses.activate
    def test_multicast_image_carousel_template_message(self):
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/multicast',
            json={}, status=200
        )

        self.tested.multicast(['to1', 'to2'], self.image_carousel_template_message)

        request = responses.calls[0].request
        self.assertEqual(request.method, 'POST')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/multicast')
        self.assertEqual(
            json.loads(request.body),
            {
                "to": ['to1', 'to2'],
                'notificationDisabled': False,
                "messages": self.image_carousel_message
            }
        )


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the 'License'); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an 'AS IS' BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from __future__ import unicode_literals, absolute_import

import unittest

from linebot.models import (
    ImagemapArea,
    ImagemapSendMessage,
    URIImagemapAction,
    MessageImagemapAction,
    Video, BaseSize, ExternalLink,
)
from tests.models.serialize_test_case import SerializeTestCase


class TestImageMap(SerializeTestCase):
    def test_image_map(self):
        arg = {
            'base_url': 'https://example.com/bot/images/rm001',
            'alt_text': 'This is an imagemap',
            'base_size': BaseSize(width=1040, height=1040),
            'video':
                Video(
                    original_content_url='https://example.com/video.mp4',
                    preview_image_url='https://example.com/video_preview.jpg',
                    area=ImagemapArea(x=0, y=0, width=1040, height=585),
                    external_link=ExternalLink(label='See more',
                                               link_uri='https://example.com/see_more.html')
                ),
            'actions': [
                URIImagemapAction(
                    link_uri='https://example.com/',
                    area=ImagemapArea(x=0, y=585, width=520, height=454)
                ),
                MessageImagemapAction(
                    text='Hey',
                    area=ImagemapArea(x=0, y=58, width=52, height=40)
                )
            ]
        }
        self.assertEqual(
            self.serialize_as_dict(arg, type=self.IMAGEMAP),
            ImagemapSendMessage(**arg).as_json_dict()
        )

    def test_video(self):
        arg = {
            'original_content_url': 'https://example.com/video.mp4',
            'preview_image_url': 'https://example.com/video_preview.jpg',
            'area': ImagemapArea(x=0, y=0, width=1040, height=585),
            'external_link': ExternalLink(label='See more',
                                          link_uri='https://example.com/see_more.html')
        }
        self.assertEqual(
            self.serialize_as_dict(arg),
            Video(**arg).as_json_dict()
        )

    def test_message_actions(self):
        arg = {
            'text': 'Hey',
            'area': ImagemapArea(x=0, y=58, width=52, height=40)
        }
        self.assertEqual(
            self.serialize_as_dict(arg, type=self.MESSAGE),
            MessageImagemapAction(**arg).as_json_dict()
        )

    def test_uri_actions(self):
        arg = {
            'link_uri': 'https://example.com/',
            'area': ImagemapArea(x=0, y=585, width=520, height=454)
        }
        self.assertEqual(
            self.serialize_as_dict(arg, self.URI),
            URIImagemapAction(**arg).as_json_dict()
        )

    def test_base_size(self):
        arg = {
            'width': 1040,
            'height': 1040
        }
        self.assertEqual(
            self.serialize_as_dict(arg),
            BaseSize(**arg).as_json_dict()
        )

    def test_external_link(self):
        arg = {
            'label': 'Hey link',
            'link_uri': 'https://example.com/see_more.html'
        }
        self.assertEqual(
            self.serialize_as_dict(arg),
            ExternalLink(**arg).as_json_dict()
        )

    def test_imagemap_area(self):
        arg = {
            'x': 111,
            'y': 33,
            'width': 1040,
            'height': 1040
        }
        self.assertEqual(
            self.serialize_as_dict(arg),
            ImagemapArea(**arg).as_json_dict()
        )


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic.v1 import BaseModel, Field, StrictStr, conlist
from linebot.v3.messaging.models.error_detail import ErrorDetail
from linebot.v3.messaging.models.sent_message import SentMessage

class ErrorResponse(BaseModel):
    """
    ErrorResponse
    https://developers.line.biz/en/reference/messaging-api/#error-responses
    """
    message: StrictStr = Field(..., description="Message containing information about the error.")
    details: Optional[conlist(ErrorDetail)] = Field(None, description="An array of error details. If the array is empty, this property will not be included in the response.")
    sent_messages: Optional[conlist(SentMessage, max_items=5, min_items=1)] = Field(None, alias="sentMessages", description="Array of sent messages.")

    __properties = ["message", "details", "sentMessages"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ErrorResponse:
        """Create an instance of ErrorResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic.v1 by calling `to_dict()` of each item in details (list)
        _items = []
        if self.details:
            for _item in self.details:
                if _item:
                    _items.append(_item.to_dict())
            _dict['details'] = _items
        # override the default output from pydantic.v1 by calling `to_dict()` of each item in sent_messages (list)
        _items = []
        if self.sent_messages:
            for _item in self.sent_messages:
                if _item:
                    _items.append(_item.to_dict())
            _dict['sentMessages'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> ErrorResponse:
        """Create an instance of ErrorResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return ErrorResponse.parse_obj(obj)

        _obj = ErrorResponse.parse_obj({
            "message": obj.get("message"),
            "details": [ErrorDetail.from_dict(_item) for _item in obj.get("details")] if obj.get("details") is not None else None,
            "sent_messages": [SentMessage.from_dict(_item) for _item in obj.get("sentMessages")] if obj.get("sentMessages") is not None else None
        })
        return _obj




# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

"""linebot.models.error module."""


from .base import Base


class Error(Base):
    """Error response of LINE messaging API.

    https://developers.line.biz/en/reference/messaging-api/#error-response
    """

    def __init__(self, message=None, details=None, **kwargs):
        """__init__ method.

        :param str message: Summary of the error
        :param details: ErrorDetail instance list
        :type details: list[T <= :py:class:`linebot.models.error.ErrorDetail`]
        :param kwargs:
        """
        super(Error, self).__init__(**kwargs)

        self.message = message

        new_details = []
        if details:
            for detail in details:
                new_details.append(
                    self.get_or_new_from_json_dict(detail, ErrorDetail)
                )
        self.details = new_details


class ErrorDetail(Base):
    """ErrorDetail response of LINE messaging API."""

    def __init__(self, message=None, property=None, **kwargs):
        """__init__ method.

        :param str message: Details of the error message
        :param str property: Related property
        :param kwargs:
        """
        super(ErrorDetail, self).__init__(**kwargs)

        self.message = message
        self.property = property



# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic.v1 import StrictStr, validator
from linebot.v3.messaging.models.flex_component import FlexComponent

class FlexSpan(FlexComponent):
    """
    FlexSpan
    """
    text: Optional[StrictStr] = None
    size: Optional[StrictStr] = None
    color: Optional[StrictStr] = None
    weight: Optional[StrictStr] = None
    style: Optional[StrictStr] = None
    decoration: Optional[StrictStr] = None
    type: str = "span"

    __properties = ["type", "text", "size", "color", "weight", "style", "decoration"]

    @validator('weight')
    def weight_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('regular', 'bold'):
            raise ValueError("must be one of enum values ('regular', 'bold')")
        return value

    @validator('style')
    def style_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('normal', 'italic'):
            raise ValueError("must be one of enum values ('normal', 'italic')")
        return value

    @validator('decoration')
    def decoration_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('none', 'underline', 'line-through'):
            raise ValueError("must be one of enum values ('none', 'underline', 'line-through')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> FlexSpan:
        """Create an instance of FlexSpan from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> FlexSpan:
        """Create an instance of FlexSpan from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return FlexSpan.parse_obj(obj)

        _obj = FlexSpan.parse_obj({
            "type": obj.get("type"),
            "text": obj.get("text"),
            "size": obj.get("size"),
            "color": obj.get("color"),
            "weight": obj.get("weight"),
            "style": obj.get("style"),
            "decoration": obj.get("decoration")
        })
        return _obj




# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the 'License'); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an 'AS IS' BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from __future__ import unicode_literals, absolute_import

import unittest

from linebot.models import (
    TemplateSendMessage,
    CarouselTemplate,
    CarouselColumn,
    ImageCarouselTemplate,
    ImageCarouselColumn,
    ConfirmTemplate,
    ButtonsTemplate,
    PostbackAction,
    URIAction,
    MessageAction,
)
from tests.models.serialize_test_case import SerializeTestCase


class TestTemplate(SerializeTestCase):
    def test_template(self):
        arg = {
            'type': 'template',
            'alt_text': 'This is a buttons template',
            'template':
                ButtonsTemplate(
                    thumbnail_image_url='https=//example.com/bot/images/image.jpg',
                    image_aspect_ratio='rectangle',
                    image_size='cover',
                    image_background_color='#FFFFFF',
                    title='Menu',
                    text='Please select',
                    default_action=URIAction(label='View detail',
                                             uri='http://example.com/page/123'),
                    actions=[
                        PostbackAction(label='Buy', data='action=buy&itemid=155'),
                        URIAction(label='View detail', uri='http://example.com/page/155'),
                    ]
                )
        }
        self.assertEqual(
            self.serialize_as_dict(arg, type=self.TEMPLATE),
            TemplateSendMessage(**arg).as_json_dict()
        )

    def test_button_template(self):
        arg = {
            'type': 'buttons',
            'thumbnail_image_url': 'https=//example.com/bot/images/image.jpg',
            'image_aspect_ratio': 'rectangle',
            'image_size': 'cover',
            'image_background_color': '#FFFFFF',
            'title': 'Menu',
            'text': 'Please select',
            'default_action': URIAction(label='View detail',
                                        uri='http://example.com/page/123'),
            'actions': [
                PostbackAction(label='Buy', data='action=buy&itemid=155'),
                URIAction(label='View detail', uri='http://example.com/page/155'),
            ]
        }
        self.assertEqual(
            self.serialize_as_dict(arg, type=self.BUTTONS),
            ButtonsTemplate(**arg).as_json_dict()
        )

    def test_confirm_template(self):
        arg = {
            'type': 'confirm',
            'text': 'Are you sure?',
            'actions': [
                MessageAction(label='Yes', text='yes...'),
                MessageAction(label='No', text='no...'),
            ]
        }
        self.assertEqual(
            self.serialize_as_dict(arg, type=self.CONFIRM),
            ConfirmTemplate(**arg).as_json_dict()
        )

    def test_carousel_template(self):
        arg = {
            'image_aspect_ratio': 'rectangle',
            'image_size': 'cover',
            'columns': [
                CarouselColumn(
                    thumbnail_image_url='https://example.com/bot/images/item1.jpg',
                    image_background_color='#FFFFFF',
                    title='this is menu',
                    text='description',
                    default_action=URIAction(label='View detail',
                                             uri='http://example.com/page/123'),
                    actions=[
                        PostbackAction(label='Buy', data='action=buy&itemid=155'),
                        URIAction(label='View detail', uri='http://example.com/page/155'),
                    ]
                ),
                CarouselColumn(
                    thumbnail_image_url='https://example.com/bot/images/item2.jpg',
                    image_background_color='#000000',
                    title='this is menu',
                    text='description',
                    default_action=URIAction(label='View detail',
                                             uri='http://example.com/page/222'),
                    actions=[
                        PostbackAction(label='Buy', data='action=buy&itemid=555'),
                        URIAction(label='View detail', uri='http://example.com/page/555'),
                    ]
                )
            ]
        }
        self.assertEqual(
            self.serialize_as_dict(arg, type=self.CAROUSEL),
            CarouselTemplate(**arg).as_json_dict()
        )

    def test_carousel_column(self):
        arg = {
            'thumbnail_image_url': 'https://example.com/bot/images/item1.jpg',
            'image_background_color': '#FFFFFF',
            'title': 'this is menu',
            'text': 'description',
            'default_action': URIAction(label='View detail',
                                        uri='http://example.com/page/123'),
            'actions': [
                PostbackAction(label='Buy', data='action=buy&itemid=155'),
                URIAction(label='View detail', uri='http://example.com/page/155'),
            ]
        }
        self.assertEqual(
            self.serialize_as_dict(arg),
            CarouselColumn(**arg).as_json_dict()
        )

    def test_image_carousel_template(self):
        arg = {
            'columns': [
                ImageCarouselColumn(
                    image_url='https://example.com/bot/images/item1.jpg',
                    action=PostbackAction(label='Buy', data='action=buy&itemid=555')
                ),
                ImageCarouselColumn(
                    image_url='https://example.com/bot/images/item2.jpg',
                    action=MessageAction(label='Yes', text='yes')
                ),
            ]
        }
        self.assertEqual(
            self.serialize_as_dict(arg, type=self.IMAGE_CAROUSEL),
            ImageCarouselTemplate(**arg).as_json_dict()
        )

    def test_image_carousel_column(self):
        arg = {
            'image_url': 'https://example.com/bot/images/item1.jpg',
            'action': PostbackAction(label='Buy', data='action=buy&itemid=555')
        }
        self.assertEqual(
            self.serialize_as_dict(arg),
            ImageCarouselColumn(**arg).as_json_dict()
        )


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic.v1 import Field, StrictStr, conlist
from linebot.v3.messaging.models.carousel_column import CarouselColumn
from linebot.v3.messaging.models.template import Template

class CarouselTemplate(Template):
    """
    CarouselTemplate
    """
    columns: conlist(CarouselColumn) = Field(...)
    image_aspect_ratio: Optional[StrictStr] = Field(None, alias="imageAspectRatio")
    image_size: Optional[StrictStr] = Field(None, alias="imageSize")
    type: str = "carousel"

    __properties = ["type", "columns", "imageAspectRatio", "imageSize"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> CarouselTemplate:
        """Create an instance of CarouselTemplate from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic.v1 by calling `to_dict()` of each item in columns (list)
        _items = []
        if self.columns:
            for _item in self.columns:
                if _item:
                    _items.append(_item.to_dict())
            _dict['columns'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> CarouselTemplate:
        """Create an instance of CarouselTemplate from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return CarouselTemplate.parse_obj(obj)

        _obj = CarouselTemplate.parse_obj({
            "type": obj.get("type"),
            "columns": [CarouselColumn.from_dict(_item) for _item in obj.get("columns")] if obj.get("columns") is not None else None,
            "image_aspect_ratio": obj.get("imageAspectRatio"),
            "image_size": obj.get("imageSize")
        })
        return _obj




# coding: utf-8

"""
    LINE Messaging API(Insight)

    This document describes LINE Messaging API(Insight).  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional, Union
from pydantic.v1 import BaseModel, Field, StrictFloat, StrictInt, StrictStr, validator

class AgeTile(BaseModel):
    """
    AgeTile
    """
    age: Optional[StrictStr] = Field(None, description="users' age")
    percentage: Optional[Union[StrictFloat, StrictInt]] = Field(None, description="Percentage")

    __properties = ["age", "percentage"]

    @validator('age')
    def age_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('from0to14', 'from15to19', 'from20to24', 'from25to29', 'from30to34', 'from35to39', 'from40to44', 'from45to49', 'from50', 'from50to54', 'from55to59', 'from60to64', 'from65to69', 'from70', 'unknown'):
            raise ValueError("must be one of enum values ('from0to14', 'from15to19', 'from20to24', 'from25to29', 'from30to34', 'from35to39', 'from40to44', 'from45to49', 'from50', 'from50to54', 'from55to59', 'from60to64', 'from65to69', 'from70', 'unknown')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> AgeTile:
        """Create an instance of AgeTile from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> AgeTile:
        """Create an instance of AgeTile from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return AgeTile.parse_obj(obj)

        _obj = AgeTile.parse_obj({
            "age": obj.get("age"),
            "percentage": obj.get("percentage")
        })
        return _obj

